/***************************************************************************
                                  Geometry.h 
                             -------------------
    begin                : 2001
    copyright            : (C)2001-2003 Multi-Agent Systems Laboratory,
                           University of Science and Technology of China 
                           Zhanxiang Huang, Yang Yang and Xiaoping Chen.
    email                : huangzx@mail.ustc.edu.cn
 ***************************************************************************/

#ifndef _GROMETRY_H_
#define _GEOMETRY_H_

#include <iostream>
#include <string>
#include <math.h>
#include <cstdio>                      // for sprintf
#include <cmath>

using namespace std;

typedef double AngRad;  // Type definition for angles in degrees. 
typedef double AngDeg;  // Type definition for angles in radians. 

const double EPSILON = 0.000001;

// auxiliary goniometric functions which enable you to
// specify angles in degrees rather than in radians
AngDeg Rad2Deg ( AngRad x             );
AngRad Deg2Rad ( AngDeg x             );
double CosDeg  ( AngDeg x             );
double SinDeg  ( AngDeg x             );
double TanDeg  ( AngDeg x             );
AngDeg ATanDeg ( double x             );
double Atan2Deg( double x,  double y  );
AngDeg ACosDeg ( double x             );
AngDeg ASinDeg ( double x             );

// auxiliary numeric functions for deterMining the
// Maximum and Minimum of two given double values and the Sign of a value
double Max     ( double d1, double d2 );
double Min     ( double d1, double d2 );
int    Sign    ( double d1            );


// matrix functions
void    MatrixMul15_51(double s1[5],double s2[5],double *d);
void    MatrixMul15_55(double s1[5],double s2[5][5],double (*d)[5]);
void    MatrixMul51_1(double s1[5],double s2,double (*d)[5]);
void    MatrixDiv51_1(double s1[5],double s2,double (*d)[5]);
void    MatrixMul51_15(double s1[5],double s2[5],double (*d)[5][5]);
void    MatrixDiv55_1(double s1[5][5],double s2,double (*d)[5][5]);
void    MatrixMul55_51(double s1[5][5],double s2[5],double (*d)[5]);
void    MatrixMul55_55(double s1[5][5],double s2[5][5],double (*d)[5][5]);
void    MatrixSub55(double s1[5][5],double s2[5][5],double (*d)[5][5]);
void    MatrixAdd51_51(double s1[5],double s2[5],double (*d)[5]);





class _Vect
{
// public methods
public:
         // constructor for _Vect class
         _Vect ( double  vx = 0,  double  vy = 0);
         _Vect ( const _Vect &v);
         
        // overloaded arithmetic operators
        virtual  _Vect        operator -           (                                 );
        virtual  _Vect        operator +          ( const double      &d            );
        virtual  _Vect        operator +          ( const _Vect &p            );
        virtual  _Vect        operator -           ( const double      &d            );
        virtual  _Vect        operator -           ( const _Vect &p            );
        virtual  _Vect        operator *           ( const double      &d            );
        virtual  _Vect        operator *           ( const _Vect &p            );
        virtual  _Vect        operator /           ( const double      &d            );
        virtual  _Vect        operator /           ( const _Vect &p            );
        virtual  void           operator =          ( const double      &d            );
        virtual  void           operator =          ( const _Vect         &v            );
        virtual  void           operator +=        ( const _Vect &p            );
        virtual  void           operator +=        ( const double      &d            );
        virtual  void           operator -=         ( const _Vect &p            );
        virtual  void           operator -=         ( const double      &d            );
        virtual  void           operator *=         ( const _Vect &p            );
        virtual  void           operator *=         ( const double      &d            );
        virtual  void           operator /=          ( const _Vect &p            );
        virtual  void           operator /=          ( const double      &d            );
        virtual  bool           operator !=          ( const _Vect &p            );
        virtual  bool           operator !=          ( const double      &d            );
        virtual  bool           operator ==         ( const _Vect &p            );
        virtual  bool           operator ==         ( const double      &d            );
        
         void  show ();
        string get_str( );

        // methods for producing output
        friend ostream&    operator<<( ostream  &os,  const _Vect  &p );


        // Set- and get methods for private member variables
        bool             SetX ( double dX);
        double        GetX ( ) const;
        bool             SetY ( double  dY );
        double        GetY ( ) const;
        void             SetVect( double dX = 0, double dY = 0);
        
          _Vect           SetMagnitude       ( double  d );
          double         GetMagnitude      (    ) const;
          AngDeg      GetDirection          (    ) const;

// private member data
protected:
         double m_x;   // x-coordinate of this position
         double m_y;   // y-coordinate of this position

        
};


class VecPos : public  _Vect{
public:
        VecPos(double x = 0, double y = 0):_Vect(x , y){}

        // Set- and get methods for derived position information
        double             GetDistanceTo    ( const VecPos p);
 
        // comparison methods for positions
        bool               IsInFrontOf  ( const VecPos &p            );
        bool               IsInFrontOf  ( const double      &d         );
        bool               IsBehindOf ( const VecPos &p            );
        bool               IsBehindOf ( const double      &d         );
        bool               IsLeftOf        ( const VecPos &p            );
        bool               IsLeftOf        ( const double      &d         );
        bool               IsRightOf     ( const VecPos &p            );
        bool               IsRightOf     ( const double      &d         ); 
        bool               IsBetweenX( const VecPos &p1, const VecPos &p2           );
        bool               IsBetweenX( const double      &d1,  const double      &d2           );
        bool               IsBetweenY( const VecPos &p1,  const VecPos &p2           );
        bool               IsBetweenY( const double      &d1,  const double      &d2           );
};


typedef  _Vect VecVel;



#endif

