/***************************************************************************
                                  Netif.C 
                             -------------------
    begin                : 2001
    copyright            : (C)2001-2003 Multi-Agent Systems Laboratory,
                           University of Science and Technology of China 
                           Zhanxiang Huang, Yang Yang and Xiaoping Chen.
    email                : huangzx@mail.ustc.edu.cn
 ***************************************************************************/

#include "Netif.h"

UDPSocket::UDPSocket()
{
}

bool UDPSocket::InitConnection(const char *SocketAddress, const unsigned  iSocketPort)
{
        struct hostent 	*host_ent ;
        struct in_addr	*addr_ptr ;

        if((host_ent = (struct hostent *)gethostbyname(SocketAddress)) == NULL) {
                // Check if it Is a numeric address
                if( inet_addr(SocketAddress) == INADDR_NONE ){
                        return false ;
                }
                else {
                        addr_ptr = (struct in_addr *) *host_ent->h_addr_list ;
                        SocketAddress = inet_ntoa(*addr_ptr) ;
                }
         }

        if( (sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
                cout << "UDPSocket::Create-->Can't create socket " << endl ;
                return false ; 
        }

        bzero( &address,sizeof(address) ) ;
        address.sin_family 	= AF_INET ;
        address.sin_addr.s_addr 	= htonl(INADDR_ANY) ;
        address.sin_port 		= htons(0) ;
	
        if( bind(sockfd, (sockaddr *)&address, sizeof(address)) < 0) {
                cout << "UDPSocket::Create-->Can't bind client to any port" << endl ;
                close(sockfd);
                return false;
        }
	
        // Set the address to the HOST 
        bzero( &address,sizeof( address ) ) ;
        address.sin_family 	= AF_INET ;
        address.sin_addr.s_addr	= inet_addr( SocketAddress ) ;
        address.sin_port	= htons( iSocketPort ) ;
        return true;
}

bool UDPSocket::InitConnection(const string &strSocketAddress, const unsigned  iSocketPort)
{
        return InitConnection(strSocketAddress.c_str(), iSocketPort);        
}

int UDPSocket::SendMsg(const string &strMsg)
{
        return SendMsg(strMsg.c_str());
}

int UDPSocket::SendMsg(const char* strMsg)
{
        int n = strlen( strMsg ) + 1 ;
        n = sendto( sockfd, strMsg, n, 0, ( sockaddr * )&address, sizeof( address ) ) ;
        return n ;
}

int UDPSocket::RecvMsg(char* strMsg)
{
        socklen_t     servlen ;
        sockaddr_in    serv_addr ;
        servlen = sizeof(serv_addr) ;
        
        int n = recvfrom( sockfd, strMsg, MAXMSG, 0, ( sockaddr * )&serv_addr, &servlen ) ;
        
        if( n > 0) {
                strMsg[n] = '\0' ;   // Set a end the to string
                address.sin_port = serv_addr.sin_port ;
        }
        
        return n ;
}

int UDPSocket::RecvMsg(string &strMsg)
{
        char msg[MAXMSG+1];
        
        int n = RecvMsg(msg);
        strMsg = msg;
        return n;
}


