/***************************************************************************
                                  TeamModel.C 
                             -------------------
    begin                : 2001
    copyright            : (C)2001-2003 Multi-Agent Systems Laboratory,
                           University of Science and Technology of China 
                           Zhanxiang Huang, Yang Yang and Xiaoping Chen.
    email                : huangzx@mail.ustc.edu.cn
 ***************************************************************************/

#include "TeamModel.h"


//**********************************************
//class Player
//**********************************************
Player::Player(){
    maxSpeed = 0;
    maxAcceleration = 0;
    maxKick = 0;
    oppDensity = 0;
    maxDribbleTime = 0;
    shootCount = 0;
    passCount = 0;
    dribbleCount = 0;
    goalCount = 0;
    role = PT_NONE;
    possessionRecord.clear();
}

bool Player::SetMaxSpeed(const double s)
{
    maxSpeed = s;
    return true;
}

bool Player::SetMaxAcceleration(const double a)
{
    maxAcceleration = a;
    return true;
}

bool Player::SetMaxKick(const double k)
{
    maxKick = k;
    return true;
}

bool Player::SetOppDensity(const double d)
{
    oppDensity = d;
    return true;
}

bool Player::SetMaxDribbleTime(const int dt)
{
    maxDribbleTime = dt;
    return true;
}

bool Player::SetShootCount(const int sc)
{
    shootCount = sc;
    return true;
}

bool Player::SetPassCount(const int pc)
{
    passCount = pc;
    return true;
}

bool Player::SetDribbleCount(const int dc)
{
    dribbleCount = dc;
    return true;
}

bool Player::SetGoalCount(const int gc)
{
    goalCount = gc;
    return true;
}

        
// interface for Getting the player parameters
double Player::GetMaxSpeed()
{
    return maxSpeed;
}

double Player::GetMaxAcceleration()
{
    return maxAcceleration;
}

double Player::GetMaxKick()
{
    return maxKick;
}

double Player::GetOppDensity()
{
    return oppDensity;
}

        
int Player::GetMaxDribbleTime()
{
    return maxDribbleTime;
}

int Player::GetShootCount()
{
    return shootCount;
}

int Player::GetPassCount()
{
    return passCount;
}

int Player::GetDribbleCount()
{
    return dribbleCount;
}

int Player::GetGoalCount()
{
    return goalCount;
}



//*************************************************************************************************************
//                                              class Formation
//*************************************************************************************************************
Formation::Formation(){
    forwardNum = 0;
    middleNum = 0;
    backNum = 0;

}


//*************************************************************************************************************
//                                              class Team
//*************************************************************************************************************
Team::Team(){
    possessionRecord.clear();
    
    //initiate the opponent position model's parameters
    for(int k=0;k<2;k++)
        for(int jj=0;jj<3;jj++)
            for(int i=0;i<MAX_PLAYER_NUMBER;i++)
                for(int j=0;j<POSITION_MODEL_PREFIX_NUMBER;j++)
                    positionModel[k][jj][i][j]=0;

    for(int i=0;i<2;i++)
        for(int jj=0;jj<3;jj++)
            for(int player=0;player<MAX_PLAYER_NUMBER;player++)
                for(int j=0;j<POSITION_MODEL_PREFIX_NUMBER;j++)
                    for(int k=0;k<POSITION_MODEL_PREFIX_NUMBER;k++)
                        if(j==k)
                            pp[i][jj][player][j][k] = MAX_NUMBER;
                        else
                            pp[i][jj][player][j][k] = 0;

    for(int i=0;i<2;i++)
        for(int jj=0;jj<3;jj++)
            e[i][jj] = 0;
}

int Team::SetWorldModelAndParameters(WorldModel* cwm, ServerSetting* css, PlayerSetting* cps, PlayerTypeSetting* cpts)
{
    wm = cwm;
    ss = css;
    ps = cps;
    pts = cpts;
    return 0;
}

void Team::SetLogger(Logger *log)
{
    l = log;
}



// This function calculate the expected position of all the players in the team
// during the interval upto current time.
int Team::UpdatePlayerExpectedPosition(SideT side, int ct, int interval)
{
    double x=0,y=0;
    f.rightestPlayerPos.SetVect(-100,0);
    f.leftestPlayerPos.SetVect(100,0);
    f.topestPlayerPos.SetVect(0,-100);
    f.bottomestPlayerPos.SetVect(0,100);

    for(int i=0;i<MAX_PLAYER_NUMBER;i++)
    {
        x=0;y=0;
        for(int j=ct;j>(ct-interval);j--)
        {
            x += wm->GetPlayerPositionX(side,i+1,j);
            y += wm->GetPlayerPositionY(side,i+1,j);
        }
        f.expectedFormation[i].SetVect( x/interval , y/interval );
        
        if( (i+1) != f.goalie )
        {
            // record the frontest mate and the tailest mate 's position    
            // exclude the goalie
            if(f.expectedFormation[i].GetX()>f.rightestPlayerPos.GetX())
                f.rightestPlayerPos = f.expectedFormation[i];
            else if(f.expectedFormation[i].GetX()<f.leftestPlayerPos.GetX())
                f.leftestPlayerPos = f.expectedFormation[i];
            if(f.expectedFormation[i].GetY()>f.bottomestPlayerPos.GetY())
                f.bottomestPlayerPos = f.expectedFormation[i];
            else if(f.expectedFormation[i].GetY()<f.topestPlayerPos.GetY())
                f.topestPlayerPos = f.expectedFormation[i];
        }
    }
    
    return 0;
}


int Player::SetRole(PlayerRoleT r)
{
    role = r;
    return 0;
}

PlayerRoleT Player::GetRole()
{
    return role;
}



PlayerActS Player::GetAction()
{
    PlayerActS actS;
    actS.actType = PAT_NONE;
    actS.actRegion.top = actS.actRegion.bottom = actS.actRegion.left = actS.actRegion.right = 0;
    return actS;
}




void Player::OutPlayerRole()
{
    switch(role)
    {
        case PT_FORWARD_LEFT:cout<<" forward_left ";break;
        case PT_FORWARD_CENTER:cout<<" forward_center ";break;
        case PT_FORWARD_RIGHT:cout<<" forward_right ";break;
        case PT_MIDFIELD_LEFT:cout<<" mid_left ";break;
        case PT_MIDFIELD_CENTER:cout<<" mid_center ";break;
        case PT_MIDFIELD_RIGHT:cout<<" mid_right ";break;
        case PT_BACK_LEFT:cout<<" back_left ";break;
        case PT_BACK_CENTER:cout<<" back_center ";break;
        case PT_BACK_RIGHT:cout<<" back_right ";break;
        case PT_GOALIE:cout<<" goalie ";break;
        default:cout<<" unknown ";break;;
    }
}



void Team::OutExpectedFormation()
{
    for(int i=0;i<MAX_PLAYER_NUMBER;i++)
    {
        cout<<" "<<f.expectedFormation[i].GetX()<<" "<<f.expectedFormation[i].GetY()<<endl;;
    }
    cout<<endl;
}


SideT Team::GetSide(TeamT t)
{
    if( t == TT_OUR )
        return wm->GetOurSide();
    else
        return wm->GetOppSide();
}


void Team::OutBallPossessionRecord()
{
    int s = possessionRecord.size();
    for(int i=0;i<s;i++)
    {
        cout<<"the "<<i+1<<" control time : "
        <<possessionRecord[i].startTime<<" "
        <<possessionRecord[i].endTime<<" "
        <<possessionRecord[i].possessionTime<<" "
        <<endl;
    }
}


void Team::UpdatePlayerPositionModel(SideT side, int ct, int interval)
{
    int suf1,suf2,suf22,count;

    x[0] = wm->GetBallPositionX(ct);
    x[1] = wm->GetBallPositionY(ct);
    x[2] = wm->GetBallVelocityX(ct);
    x[3] = wm->GetBallVelocityY(ct);
    x[4] = 1;
    //l->Log(8,"f(t) = %f\t%f\t%f\t%f\t1",x[0],x[1],x[2],x[3],x[4]);

    //set the suf2 by ball X coordinate
    if(x[0]>=FORWARD_FIELD_THRESHOLD)
        suf2 = 0;
    else if(x[0]<=BACK_FIELD_THRESHOLD)
        suf2 = 2;
    else 
        suf2 = 1;
        
    // calculate the X Y model
    for(int i=0;i<MAX_PLAYER_NUMBER;i++)
    {
        //l->Log(8,"calculate the %d opponent position model",i+1);

        for(suf1=0;suf1<2;suf1++)
        {            
            //l->Log(8,"f(t) = %f\t%f\t%f\t%f\t1",x[0],x[1],x[2],x[3],x[4]);
            //L(t) = P(t-1)f(t)/[1+fT(t)P(t-1)f(t)]
            MatrixMul55_51(pp[suf1][suf2][i],x,&(ll[suf1][suf2][i]) );
            MatrixMul15_55(x,pp[suf1][suf2][i],&(temp[suf1][suf2]) );
            MatrixMul15_51(temp[suf1][suf2],x,&(temp2[suf1][suf2]) );
            temp2[suf1][suf2] += 1;
            MatrixDiv51_1(ll[suf1][suf2][i],temp2[suf1][suf2],&(ll[suf1][suf2][i]) );
            //l->Log(8,"ok calculate L = %f\t%f\t%f\t%f\t%f\t",ll[suf1][suf2][i][0],ll[suf1][suf2][i][1],ll[suf1][suf2][i][2],ll[suf1][suf2][i][3],ll[suf1][suf2][i][4]);
            
            //l->Log(8,"P(t-1) = ");
            //for(int ii=0;ii<POSITION_MODEL_PREFIX_NUMBER;ii++)
                //l->Log(8,"%f\t%f\t%f\t%f\t%f\t",pp[suf1][suf2][i][ii][0],pp[suf1][suf2][i][ii][1],pp[suf1][suf2][i][ii][2],pp[suf1][suf2][i][ii][3],pp[suf1][suf2][i][ii][4]);
            //P(t) = P(t-1)-P(t-1)f(t)fT(t)P(t-1)/[1+fT(t)P(t-1)f(t)]
            MatrixMul55_51(pp[suf1][suf2][i],x,&(temp[suf1][suf2]) );
            //l->Log(8,"P(t-1)f(t) = ");
            //l->Log(8,"%f\t%f\t%f\t%f\t%f\t",temp[suf1][suf2][0],temp[suf1][suf2][1],temp[suf1][suf2][2],temp[suf1][suf2][3],temp[suf1][suf2][4]);
            MatrixMul51_15(temp[suf1][suf2],x,&temp3);
            //l->Log(8,"P(t-1)f(t)fT(t) = ");
            //for(int ii=0;ii<POSITION_MODEL_PREFIX_NUMBER;ii++)
                //l->Log(8,"%f\t%f\t%f\t%f\t%f\t",temp3[ii][0],temp3[ii][1],temp3[ii][2],temp3[ii][3],temp3[ii][4]);
            MatrixMul55_55(temp3,pp[suf1][suf2][i],&temp4);
            //l->Log(8,"P(t-1)f(t)fT(t)P(t-1) = ");
            //for(int ii=0;ii<POSITION_MODEL_PREFIX_NUMBER;ii++)
                //l->Log(8,"%f\t%f\t%f\t%f\t%f\t",temp4[ii][0],temp4[ii][1],temp4[ii][2],temp4[ii][3],temp4[ii][4]);
            MatrixDiv55_1(temp4,temp2[suf1][suf2],&temp4);
            //l->Log(8,"P(t-1)f(t)fT(t)P(t-1)/[1+fT(t)P(t-1)f(t)] = ");
            //for(int ii=0;ii<POSITION_MODEL_PREFIX_NUMBER;ii++)
                //l->Log(8,"%f\t%f\t%f\t%f\t%f\t",temp4[ii][0],temp4[ii][1],temp4[ii][2],temp4[ii][3],temp4[ii][4]);
            MatrixSub55(pp[suf1][suf2][i],temp4,&(pp[suf1][suf2][i]) );
            //l->Log(8,"ok calculate P = ");
            //for(int ii=0;ii<POSITION_MODEL_PREFIX_NUMBER;ii++)
                //l->Log(8,"%f\t%f\t%f\t%f\t%f\t",pp[suf1][suf2][i][ii][0],pp[suf1][suf2][i][ii][1],pp[suf1][suf2][i][ii][2],pp[suf1][suf2][i][ii][3],pp[suf1][suf2][i][ii][4]);
            
            //c(t) = c(t-1)+L(t)[y(t)-cT(t-1)f(t)]
            MatrixMul15_51(positionModel[suf1][suf2][i],x,&(temp2[suf1][suf2]) );
            if(suf1==0)
                temp2[suf1][suf2] = wm->GetPlayerPositionX(side,i+1,ct) - temp2[suf1][suf2];
            else
                temp2[suf1][suf2] = wm->GetPlayerPositionY(side,i+1,ct) - temp2[suf1][suf2];
            MatrixMul51_1(ll[suf1][suf2][i],temp2[suf1][suf2],&(temp[suf1][suf2]) );
            MatrixAdd51_51(positionModel[suf1][suf2][i],temp[suf1][suf2],&(positionModel[suf1][suf2][i]) );
            //l->Log(8,"y(t) = %f",wm->GetPlayerPositionX(side,i+1,ct) );
            //l->Log(8,"ok calculate c = %f\t%f\t%f\t%f\t%f\t",positionModel[suf1][suf2][i][0],positionModel[suf1][suf2][i][1],positionModel[suf1][suf2][i][2],positionModel[suf1][suf2][i][3],positionModel[suf1][suf2][i][4]);

            e[suf1][suf2] = 0;
            count = 0;
            for(int kk=ct;kk>0;kk--)
            {
                if(wm->GetBallPositionX(kk)>=FORWARD_FIELD_THRESHOLD)
                    suf22 = 0;
                else if(wm->GetBallPositionX(kk)<=BACK_FIELD_THRESHOLD)
                    suf22 = 2;
                else 
                    suf22 = 1;
                
                if(suf22==suf2)
                {
                    xx[0] = wm->GetBallPositionX(kk);
                    xx[1] = wm->GetBallPositionY(kk);
                    xx[2] = wm->GetBallVelocityX(kk);
                    xx[3] = wm->GetBallVelocityY(kk);
                    xx[4] = 1;
                    if(suf1==0)
                        temp5[suf1][suf2] = wm->GetPlayerPositionX(side,i+1,kk) - 
                            (xx[0]*positionModel[suf1][suf2][i][0]+xx[1]*positionModel[suf1][suf2][i][1]
                            +xx[2]*positionModel[suf1][suf2][i][2]+xx[3]*positionModel[suf1][suf2][i][3]
                            +xx[4]*positionModel[suf1][suf2][i][4]);
                    else
                        temp5[suf1][suf2] = wm->GetPlayerPositionY(side,i+1,kk) - 
                            (xx[0]*positionModel[suf1][suf2][i][0]+xx[1]*positionModel[suf1][suf2][i][1]
                            +xx[2]*positionModel[suf1][suf2][i][2]+xx[3]*positionModel[suf1][suf2][i][3]
                            +xx[4]*positionModel[suf1][suf2][i][4]);
                    temp5[suf1][suf2] *= temp5[suf1][suf2];
                    e[suf1][suf2] += temp5[suf1][suf2];
                    count++;
                }
            }
            temp5[suf1][suf2] = e[suf1][suf2];
            temp5[suf1][suf2] /= count;
            e[suf1][suf2] = sqrt(temp5[suf1][suf2]);
            //l->Log(8,"%c coordinate %c field : the %d step error is %f",suf1==0?'X':'Y',
                //suf2==0?'f':(suf2==1?'m':'b'),count,e[suf1][suf2]);
        }
        //l->Log(8,"^^^^^^^^^^^^the %d opponent position model ok!^^^^^^^^^^^^^^^",i+1);
    }
}




int Team::GetControlBallTime()
{
    int time=0;
    vector<BallPossessionStruct>::iterator iter;
    for(iter=possessionRecord.begin();iter!=possessionRecord.end();iter++)
    {
        if( (*iter).set )
            time += (*iter).possessionTime;
        else 
            l->Log(0,"!!notset!!");
        //l->Log(0,"%d %d" ,(*iter).possessionTime,time);
    }
    return time;
}

double Team::GetControlBallRatio()
{
    int time = GetControlBallTime();
    //l->Log(0,"time == %d",time);
    if(wm->GetCurrentTime()==0)
        return 0;
    else
        return float(time)/float(wm->GetCurrentTime());
}







