/***************************************************************************
                                  WECoach.C 
                             -------------------
    begin                : 2001
    copyright            : (C)2001-2003 Multi-Agent Systems Laboratory,
                           University of Science and Technology of China 
                           Zhanxiang Huang, Yang Yang and Xiaoping Chen.
    email                : huangzx@mail.ustc.edu.cn
 ***************************************************************************/

#include "WECoach.h"

using namespace std;


//*********************************************************************************
//          Construct functions
//*********************************************************************************
WECoach::WECoach():BasicCoach()
{
    l = new Logger("logfile");
    wm.SetOurSide(FIELD_SIDE_L);
    iSayInterval = 0;
    mm.SetWorldModelAndParameters(&wm, &ss, &ps, &pts);
    om.SetWorldModelAndParameters(&wm, &ss, &ps, &pts);
    mm.SetLogger(l);
    om.SetLogger(l);
    bGoalieSet = false;
    lastBallPossessionState = BP_NONE;
    currentBallPossessionState = BP_NONE;
    InitFormPara();
    bPlayerRolesSet = false;
    bDecidePlayerType = false;
    bDecidePositioning = false;
    ss.SetLogger(l);
    ps.SetLogger(l);
    pts.SetLogger(l);
    wm.SetLogger(l);
    checkPossessionRecordIndex = -1;
    receiveMsgMark = false;
    ruleName = 0;
    proccessRuleMark = 0;
    superviseRuleMark = 0;
}

WECoach::WECoach(UDPSocket *s, Parse *parse , string &ourTeamName , Logger* cl , SideT side)
        :BasicCoach(s, parse, ourTeamName)
{
    l = cl;
    wm.SetOurSide(side);
    wm.SetServerParameters(&ss, &ps, &pts);
    iSayInterval = 0;
    mm.SetWorldModelAndParameters(&wm, &ss, &ps, &pts);
    om.SetWorldModelAndParameters(&wm, &ss, &ps, &pts);
    mm.SetLogger(l);
    om.SetLogger(l);
    bGoalieSet = false;
    lastBallPossessionState = BP_NONE;
    currentBallPossessionState = BP_NONE;
    InitFormPara();
    bPlayerRolesSet = false;
    bDecidePlayerType = false;
    bDecidePositioning = false;
    ss.SetLogger(l);
    ps.SetLogger(l);
    pts.SetLogger(l);
    wm.SetLogger(l);
    checkPossessionRecordIndex = -1;
    receiveMsgMark = false;
    ruleName = 0;
    proccessRuleMark = 0;
    superviseRuleMark = 0;
}



int WECoach::InitFormPara()
{
	// ʼβ
	FILE*	fp;
	char buf[1024];

	if( (fp = fopen(FORM_PARA,"r")) != NULL){
		// skip commence 
		// 2 player
		while(fgets(buf,1024,fp)!=NULL)	if(buf[0]!='#' && strlen(buf)!=0) break;
		sscanf(buf,"%d %d %d %d %d %d",&H2[0][0],&H2[0][1],&H2[1][0],&H2[1][1],&H2[2][0],&H2[2][1]);
		// 2 player
		while(fgets(buf,1024,fp)!=NULL)	if(buf[0]!='#' && strlen(buf)!=0) break;
		sscanf(buf,"%d %d %d %d %d %d",&V2[0][0],&V2[0][1],&V2[1][0],&V2[1][1],&V2[2][0],&V2[2][1]);
		// 3 player	
		while(fgets(buf,1024,fp)!=NULL)	if(buf[0]!='#' && strlen(buf)!=0) break;
		sscanf(buf,"%d %d %d %d %d %d %d %d %d",&H3[0][0],&H3[0][1],&H3[0][2],&H3[1][0],&H3[1][1],
			&H3[1][2],&H3[2][0],&H3[2][1],&H3[2][2]);
		// 3 player
		while(fgets(buf,1024,fp)!=NULL)	if(buf[0]!='#' && strlen(buf)!=0) break;
		sscanf(buf,"%d %d %d %d %d %d %d %d %d",&V3[0][0],&V3[0][1],&V3[0][2],&V3[1][0],&V3[1][1],
			&V3[1][2],&V3[2][0],&V3[2][1],&V3[2][2]);
		// 4 player
		while(fgets(buf,1024,fp)!=NULL)	if(buf[0]!='#' && strlen(buf)!=0) break;
		sscanf(buf,"%d %d %d %d %d %d %d %d %d %d %d %d",&H4[0][0],&H4[0][1],&H4[0][2],&H4[0][3],&H4[1][0],
			&H4[1][1],&H4[1][2],&H4[1][3],&H4[2][0],&H4[2][1],&H4[2][2],&H4[2][3]);
		// 4 player
		while(fgets(buf,1024,fp)!=NULL)	if(buf[0]!='#' && strlen(buf)!=0) break;
		sscanf(buf,"%d %d %d %d %d %d %d %d %d %d %d %d",&V4[0][0],&V4[0][1],&V4[0][2],&V4[0][3],&V4[1][0],
			&V4[1][1],&V4[1][2],&V4[1][3],&V4[2][0],&V4[2][1],&V4[2][2],&V4[2][3]);
		// 5 player
		while(fgets(buf,1024,fp)!=NULL)	if(buf[0]!='#' && strlen(buf)!=0) break;
		sscanf(buf,"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",&H5[0][0],&H5[0][1],&H5[0][2],&H5[0][3],&H5[0][4],
			&H5[1][0],&H5[1][1],&H5[1][2],&H5[1][3],&H5[1][4],&H5[2][0],&H5[2][1],&H5[2][2],&H5[2][3],&H5[2][4]);
		// 5 player
		while(fgets(buf,1024,fp)!=NULL)	if(buf[0]!='#' && strlen(buf)!=0) break;
		sscanf(buf,"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",&V5[0][0],&V5[0][1],&V5[0][2],&V5[0][3],&V5[0][4],
			&V5[1][0],&V5[1][1],&V5[1][2],&V5[1][3],&V5[1][4],&V5[2][0],&V5[2][1],&V5[2][2],&V5[2][3],&V5[2][4]);
		while(fgets(buf,1024,fp)!=NULL)	if(buf[0]!='#' && strlen(buf)!=0) break;
        sscanf(buf,"%d %d %d",&forwardLine,&middleLine,&backLine);
	    fclose(fp);
	}
	else
	    return -1;
    
    return 0;
}






//*********************************************************************************
//          Update World Model
//*********************************************************************************

// This functio execute the routine of updating the worldmodel each cycle
int WECoach::UpdateRoutine()
{
    MsgState msgs = (MsgState)BasicCoach::UpdateRoutine();
    if( msgs != MsgS_See_Msg_Come );//ûϢ//return 0;
    return 1;
}






//*********************************************************************************
//          Send Commands
//*********************************************************************************

// This function Is charge for sending commands to server
// iSayInterval Is maintained in this function
int WECoach::SendCommand()
{
    int res;
    bool bCanSay = false;
    
    if( iSayInterval>0 ) iSayInterval--;  // decrease the iSayInterval

    PlayModeT pm = wm.GetPlayMode(wm.GetCurrentTime());
    if( //pm == PM_BEFORE_KICK_OFF    ||  pm == PM_KICK_OFF_LEFT      ||  pm == PM_KICK_OFF_RIGHT     || 
        //pm == PM_KICK_IN_LEFT       ||  pm == PM_KICK_IN_RIGHT      ||  
        //pm == PM_CORNER_KICK_LEFT   ||  pm == PM_CORNER_KICK_RIGHT  ||  
        //pm == PM_GOAL_KICK_LEFT     ||  pm == PM_GOAL_KICK_RIGHT    || 
        //pm == PM_FREE_KICK_LEFT     ||  pm == PM_FREE_KICK_RIGHT    
        pm != PM_PLAY_ON
        || iSayInterval == 0
      )
        bCanSay = true;

    //l->Log(4,"pm = %s , iSayInterval = %d", pm==PM_PLAY_ON?"play_on":"not playon", iSayInterval);

    if( !bCanSay ) return 0;            // it Is not time to send commands
    if( sendMsgQ.size()==0 ) return 0;    // the command queue Is empty
    
    list<Command>::iterator  iter;
    string token;

    while( sendMsgQ.size()>0 )
    {
        iter = sendMsgQ.begin();
        switch( (*iter).type )
        {
            case CT_COMMAND:
                res = Send( (*iter).c);
                l->Log(4,"at time %d Sent command %s ", wm.GetCurrentTime(),(*iter).c.c_str());
                sendMsgQ.pop_front();
                break;
            case CT_SAY:
                do
                {
                    res = Say( (*iter).c);
                    l->Log(4,"at time %d Sent say %s ", wm.GetCurrentTime(),(*iter).c.c_str());
                    sendMsgQ.pop_front();
                    iter = sendMsgQ.begin();
                }while(sendMsgQ.size()!=0 && (*iter).type == CT_SAY);
                break;
            case CT_TOKEN:
                token = "(advice";
                while( (*iter).type==CT_TOKEN && sendMsgQ.size()>0 && token.length()<MAX_TOKEN_LENGTH )
                {
                    iter = sendMsgQ.begin();
                    token += " " + (*iter).c;
                    sendMsgQ.pop_front();
                    if(sendMsgQ.size()==0)
                        break;
                    else
                        iter = sendMsgQ.begin();
                }
                token += ")";
                res = Say(token);
                l->Log(4,"at time %d Sent token %s ", wm.GetCurrentTime(),token.c_str());
                break;
             case CT_DEFINE:
                do
                {
                    iter = sendMsgQ.begin();
                    if( (*iter).type!=CT_DEFINE ) break;
                    token = "(define";
                    do
                    {
                        if( (*iter).createTime < (wm.GetCurrentTime() - ADVICE_LIVE_TIME_NORMAL) )
                        {
                            //token += " " + (*iter).c;
                            sendMsgQ.pop_front();
                            ruleRecordQ[proccessRuleMark].onTime = 0;
                            ruleRecordQ[proccessRuleMark].sent = false;
                            ruleRecordQ[proccessRuleMark].on = false;
                            proccessRuleMark++;
                            iter = sendMsgQ.begin();
                        }
                        else
                        {
                            token += " " + (*iter).c;
                            sendMsgQ.pop_front();
                            ruleRecordQ[proccessRuleMark].onTime = wm.GetCurrentTime();
                            ruleRecordQ[proccessRuleMark].sent = true;
                            ruleRecordQ[proccessRuleMark].on = false;
                            proccessRuleMark++;
                            iter = sendMsgQ.begin();
                        }
                    }while(sendMsgQ.size() != 0 && (*iter).type == CT_DEFINE && token.length()<MAX_TOKEN_LENGTH);
                    token += ")";
                    res = Say(token);
                    l->Log(4,"at time %d Sent %s ", wm.GetCurrentTime(),token.c_str());
                }while(sendMsgQ.size()!=0 && pm!=PM_PLAY_ON);
                break;
            default:
                res = 0;
        };
        //if(iter==sendMsgQ.end() ) break;
        if(pm == PM_PLAY_ON) break;
    }//end while
    
    //Say("(rule (on all))");
    iSayInterval = MIN_INTERVAL;
    
    return res;
}


string WECoach::ChangePlayerTypeCommand(int unum, int playerType)
{
    if(playerType == 0) return "";
    string res, bufUnum, bufType;
    char temp[10];
    sprintf(temp, "%d" , unum);
    bufUnum = temp;
    sprintf(temp, "%d" , playerType);
    bufType = temp;
    res = "(change_player_type " + bufUnum + " " + bufType + ")";
    return res;
}



//*********************************************************************************************
// fm : formation 
// backLineXStyle: з backLineYStyle:з backLine: 
// middleLineXStyle: гз middleLineYStyle:гз middleLine: г
// forwardLineXStyle: ǰз forwardLineYStyle:ǰз forwardLine: ǰ
//*********************************************************************************************
string WECoach::HomeCommand(FormationT fm,int backLineXStyle,int backLineYStyle,int backLine,int middleLineXStyle,int middleLineYStyle,int middleLine,int forwardLineXStyle,int forwardLineYStyle,int forwardLine)
{
    int i;
    char FormationBuf[1024];

	VecPos hpos[TEAM_SIZE];  // store hpos
	//buf="(info (1 (true) (do our {1} (home (pt -45 0)))) (1 (true) (do our {2} (home (pt -8 -21)))) (1 (true) (do our {3} (home (pt -15 -7)))) (1 (true) (do our {4} (home (pt -15 7)))) (1 (true) (do our {5} (home (pt -8 21)))) (1 (true) (do our {6} (home (pt 0 0)))) (1 (true) (do our {7} (home (pt 5 -7)))) (1 (true) (do our {8} (home (pt 5 7)))) (1 (true) (do our {9} (home (pt 15 0)))) (1 (true) (do our {10} (home (pt 20 -20)))) (1 (true) (do our {11} (home (pt 20 20)))) )";

	// goalie home pos
	//hpos[1].SetX( mm.GetSide(TT_OUR)==FIELD_SIDE_L?-45:45 );
	//hpos[1].SetY( 0 );

	switch(fm){
	case FT_433:
		// defender No.2-5
		for(i=2; i<=5; i++){
			hpos[i].SetX( backLine + H4[backLineXStyle][i-2] );
			hpos[i].SetY( V4[backLineYStyle][i-2] );
		}
		// center No.6-8
		for(i=6; i<=8; i++){
			hpos[i].SetX( middleLine + H3[middleLineXStyle][i-6] );
			hpos[i].SetY( V3[middleLineYStyle][i-6] );
		}
		// forward No.9-11
		for(i=9; i<=11; i++){
			hpos[i].SetX( forwardLine + H3[forwardLineXStyle][i-9] );
			hpos[i].SetY( V3[forwardLineYStyle][i-9] );
		}
		break;
	case FT_442:
		// defender No.2-5
		for(i=2; i<=5; i++){
			hpos[i].SetX( backLine + H4[backLineXStyle][i-2] );
			hpos[i].SetY( V4[backLineYStyle][i-2] );
		}
		// center No.6-9
		for(i=6; i<=9; i++){
			hpos[i].SetX( middleLine + H4[middleLineXStyle][i-6] );
			hpos[i].SetY( V4[middleLineYStyle][i-6] );
		}
		// forward No.10-11
		for(i=10; i<=11; i++){
			hpos[i].SetX( forwardLine + H2[forwardLineXStyle][i-10] );
			hpos[i].SetY( V2[forwardLineYStyle][i-10] );
		}
		break;
	case FT_451:
		// defender No.2-5
		for(i=2; i<=5; i++){
			hpos[i].SetX( backLine + H4[backLineXStyle][i-2] );
			hpos[i].SetY( V4[backLineYStyle][i-2] );
		}
		// center No.6-10
		for(i=6; i<=10; i++){
			hpos[i].SetX( middleLine + H5[middleLineXStyle][i-6] );
			hpos[i].SetY( V5[middleLineYStyle][i-6] );
		}
		// forward No.11
		hpos[11].SetX( forwardLine );
		hpos[11].SetY( 0 );
		break;
	case FT_424:
		// defender No.2-5
		for(i=2; i<=5; i++){
			hpos[i].SetX( backLine + H4[backLineXStyle][i-2] );
			hpos[i].SetY( V4[backLineYStyle][i-2] );
		}
		// center No.6-7
		for(i=6; i<=7; i++){
			hpos[i].SetX( middleLine + H2[middleLineXStyle][i-6] );
			hpos[i].SetY( V2[middleLineYStyle][i-6] );
		}
		// forward No.8-11
		for(i=8; i<=11; i++){
			hpos[i].SetX( forwardLine + H4[forwardLineXStyle][i-8] );
			hpos[i].SetY( V4[forwardLineYStyle][i-8] );
		}
		break;
	case FT_352:
		// defender No.2-4
		for(i=2; i<=4; i++){
			hpos[i].SetX( backLine + H3[backLineXStyle][i-2] );
			hpos[i].SetY( V3[backLineYStyle][i-2] );
		}
		// center No.5-9
		for(i=5; i<=9; i++){
			hpos[i].SetX( middleLine + H5[middleLineXStyle][i-5] );
			hpos[i].SetY( V5[middleLineYStyle][i-5] );
		}
		// forward No.10-11
		for(i=10; i<=11; i++){
			hpos[i].SetX( forwardLine + H2[forwardLineXStyle][i-10] );
			hpos[i].SetY( V2[forwardLineYStyle][i-10] );
		}
		break;
	case FT_343:
		// defender No.2-4
		for(i=2; i<=4; i++){
			hpos[i].SetX( backLine + H3[backLineXStyle][i-2] );
			hpos[i].SetY( V3[backLineYStyle][i-2] );
		}
		// center No.5-8
		for(i=5; i<=8; i++){
			hpos[i].SetX( middleLine + H4[middleLineXStyle][i-5] );
			hpos[i].SetY( V4[middleLineYStyle][i-5] );
		}
		// forward No.9-11
		for(i=9; i<=11; i++){
			hpos[i].SetX( forwardLine + H3[forwardLineXStyle][i-9] );
			hpos[i].SetY( V3[forwardLineYStyle][i-9] );
		}
		break;
	case FT_334:
		// defender No.2-4
		for(i=2; i<=4; i++){
			hpos[i].SetX( backLine + H3[backLineXStyle][i-2] );
			hpos[i].SetY( V3[backLineYStyle][i-2] );
		}
		// center No.5-7
		for(i=5; i<=7; i++){
			hpos[i].SetX( middleLine + H3[middleLineXStyle][i-5] );
			hpos[i].SetY( V3[middleLineYStyle][i-5] );
		}
		// forward No.8-11
		for(i=8; i<=11; i++){
			hpos[i].SetX( forwardLine + H4[forwardLineXStyle][i-8] );
			hpos[i].SetY( V4[forwardLineYStyle][i-8] );
		}
		break;
	case FT_532:
		// defender No.2-6
		for(i=2; i<=6; i++){
			hpos[i].SetX( backLine + H5[backLineXStyle][i-2] );
			hpos[i].SetY( V5[backLineYStyle][i-2] );
		}
		// center No.7-9
		for(i=7; i<=9; i++){
			hpos[i].SetX( middleLine + H3[middleLineXStyle][i-7] );
			hpos[i].SetY( V3[middleLineYStyle][i-7] );
		}
		// forward No.10-11
		for(i=10; i<=11; i++){
			hpos[i].SetX( forwardLine + H2[forwardLineXStyle][i-10] );
			hpos[i].SetY( V2[forwardLineYStyle][i-10] );
		}
		break;
	default:
	    return "";
	    break;
	};

	sprintf(FormationBuf,"(definerule %s direc ((bowner our { 1 2 3 4 5 6 7 8 9 10 }) (do our {%d} (home (pt %.1f %.1f))) (do our {%d} (home (pt %.1f %.1f))) (do our {%d} (home (pt %.1f %.1f))) (do our {%d} (home (pt %.1f %.1f))) (do our {%d} (home (pt %.1f %.1f))) (do our {%d} (home (pt %.1f %.1f))) (do our {%d} (home (pt %.1f %.1f))) (do our {%d} (home (pt %.1f %.1f))) (do our {%d} (home (pt %.1f %.1f))) (do our {%d} (home (pt %.1f %.1f)))))",
        GetRuleName().c_str(),
		/*mm.role2PlayerMap[0], hpos[1].GetX(), hpos[1].GetY(),*/
	    mm.role2PlayerMap[1], hpos[2].GetX(), hpos[2].GetY(),
	    mm.role2PlayerMap[2],  hpos[3].GetX(), hpos[3].GetY(),
	    mm.role2PlayerMap[3],  hpos[4].GetX(), hpos[4].GetY(),
	    mm.role2PlayerMap[4], hpos[5].GetX(), hpos[5].GetY(),
	    mm.role2PlayerMap[5],  hpos[6].GetX(), hpos[6].GetY(),
	    mm.role2PlayerMap[6], hpos[7].GetX(), hpos[7].GetY(),
	    mm.role2PlayerMap[7],  hpos[8].GetX(), hpos[8].GetY(),
	    mm.role2PlayerMap[8],  hpos[9].GetX(), hpos[9].GetY(),
	    mm.role2PlayerMap[9], hpos[10].GetX(), hpos[10].GetY(),
	    mm.role2PlayerMap[10], hpos[11].GetX(), hpos[11].GetY()
	  );
    
    string res = FormationBuf;
    
    return res;
}


int WECoach::Int2RegionChar(int x)
{
    if(x<0||x>volNum) x = volNum;
    return (x>9?(x+87):(x+48));
}


string WECoach::PlayMode2String(PlayModeT pm)
{
    string r = "(playm ";//(playm <PLAY_MODE>)

    switch(pm)
    {
        case PM_BEFORE_KICK_OFF     :   r += "bko)";    break;
        case PM_KICK_OFF_LEFT       :   r += (wm.GetOppSide()==FIELD_SIDE_L?"ko_our)":"ko_opp)"); break;
        case PM_KICK_OFF_RIGHT      :   r += (wm.GetOppSide()==FIELD_SIDE_R?"ko_our)":"ko_opp)"); break;
        case PM_KICK_IN_LEFT        :   r += (wm.GetOppSide()==FIELD_SIDE_L?"ki_our)":"ki_opp)"); break;
        case PM_KICK_IN_RIGHT       :   r += (wm.GetOppSide()==FIELD_SIDE_R?"ki_our)":"ki_opp)"); break;
        case PM_CORNER_KICK_LEFT    :   r += (wm.GetOppSide()==FIELD_SIDE_L?"ck_our)":"ck_opp)"); break;
        case PM_CORNER_KICK_RIGHT   :   r += (wm.GetOppSide()==FIELD_SIDE_R?"ck_our)":"ck_opp)"); break;
        case PM_GOAL_KICK_LEFT      :   r += (wm.GetOppSide()==FIELD_SIDE_L?"gk_our)":"gk_opp)"); break;
        case PM_GOAL_KICK_RIGHT     :   r += (wm.GetOppSide()==FIELD_SIDE_R?"gk_our)":"gk_opp)"); break;
        case PM_FREE_KICK_LEFT      :   r += (wm.GetOppSide()==FIELD_SIDE_L?"fk_our)":"fk_opp)"); break;
        case PM_FREE_KICK_RIGHT     :   r += (wm.GetOppSide()==FIELD_SIDE_R?"fk_our)":"fk_opp)"); break;
        case PM_PLAY_ON             :   r += "play_on)";break;
        case PM_TIME_OVER           :   r += "time_over)";break;
        default:
        break;
    }
    
    return r;
}

string WECoach::BallPos2String(Region ballPos)
{
/*
    //(bpos (rec (pt [real] [real]) (pt [real] [real])))
    char temp[1024];
    string r = "(bpos (rec (pt ";
    sprintf(temp,"%0.1f %0.1f) (pt %0.1f %0.1f)))",ballPos.left,ballPos.top,ballPos.right,ballPos.bottom);
    r += temp;
    return r;
*/
    //(bpos <REGION>)
    int ballX = int( ((ballPos.left+ballPos.right+FIELD_LENGTH)/2)/volWidth );
    int ballY = int( ((ballPos.top+ballPos.bottom+FIELD_WIDTH)/2)/rowWidth );
    char temp[1024];
    sprintf(temp,"(bpos \"%c%c\")",Int2RegionChar(ballX),Int2RegionChar(ballY) );
    string r = temp;
    return r;
}


string WECoach::PlayerPos2String(TeamT team, vector<int> unum, int min, int max, Region playerPos)
{
/*
    //(ppos <TEAM> <UNUM_SET> <INT> <INT> (rec (pt [real] [real]) (pt [real] [real])))
    string r = "(ppos ";
    r += (team==TT_OUR?"our":"opp");
    r += " {";
    
    char temp[1024];
    vector<int>::iterator iter;
    for(iter=unum.begin();iter!=unum.end();iter++)
    {
        sprintf(temp," %d",(*iter) );
        r += temp;    //??????????
    }
    r += " } ";//????????????
    sprintf(temp,"%d %d (rec (pt %0.1f %0.1f) (pt %0.1f %0.1f)))",min,max,playerPos.left,playerPos.top,playerPos.right,playerPos.bottom);
    r += temp;

    return r;
*/
    //(ppos <TEAM> <UNUM_SET> <INT> <INT> <REGION>)
    string r = "(ppos ";
    r += (team==TT_OUR?"our":"opp");
    r += " {";
    
    char temp[1024];
    vector<int>::iterator iter;
    for(iter=unum.begin();iter!=unum.end();iter++)
    {
        sprintf(temp," %d",(*iter) );
        r += temp;    //??????????
    }
    r += " } ";//????????????
    int x = int( ((playerPos.left+playerPos.right+FIELD_LENGTH)/2)/volWidth );
    int y = int( ((playerPos.top+playerPos.bottom+FIELD_WIDTH)/2)/rowWidth );
    sprintf(temp,"%d %d \"%c%c\")",min,max,Int2RegionChar(x),Int2RegionChar(y) );
    r += temp;

    return r;
}

string WECoach::PlayerAction2String(PlayerActS playerAct)
{
    //(pass <REGION>)|(pos <REGION>)|(dribble <REGION>)|(shoot)
    int x,y;
    char temp[1024];
    string r = "(";
    switch(playerAct.actType)
    {
        case PAT_POS:
            x = int( ((playerAct.actRegion.left+playerAct.actRegion.right+FIELD_LENGTH)/2)/volWidth );
            y = int( ((playerAct.actRegion.top+playerAct.actRegion.bottom+FIELD_WIDTH)/2)/rowWidth );
            sprintf(temp,"pos \"%c%c\")",Int2RegionChar(x),Int2RegionChar(y) );
            r += temp;
            break;
        case PAT_PASS:
            x = int( ((playerAct.actRegion.left+playerAct.actRegion.right+FIELD_LENGTH)/2)/volWidth );
            y = int( ((playerAct.actRegion.top+playerAct.actRegion.bottom+FIELD_WIDTH)/2)/rowWidth );
            sprintf(temp,"pass \"%c%c\")",Int2RegionChar(x),Int2RegionChar(y) );
            r += temp;
            break;
        case PAT_DRIBBLE:
            x = int( ((playerAct.actRegion.left+playerAct.actRegion.right+FIELD_LENGTH)/2)/volWidth );
            y = int( ((playerAct.actRegion.top+playerAct.actRegion.bottom+FIELD_WIDTH)/2)/rowWidth );
            sprintf(temp,"dribble \"%c%c\")",Int2RegionChar(x),Int2RegionChar(y) );
            r += temp;
            break;
        case PAT_SHOOT:
            r += "shoot)";
            break;
        default:
            r += "shoot)";
            break;
    }
    
    return r;
}

string WECoach::PlayerAction2String(ActionMode mode, TeamT team, vector<int> unum, PlayerActS playerAct)
{
    //(<MODE> <TEAM> <UNUM_SET> <ACTION_LIST>)
    //(pass <REGION>)|(pos <REGION>)|(dribble <REGION>)|(shoot)
    string r = (mode==AM_DO?"(do ":"(dont ");
    r += (team==TT_OUR?"our {":"opp {");
    char temp[1024];
    vector<int>::iterator iter;
    for(iter=unum.begin();iter!=unum.end();iter++)
    {
        sprintf(temp," %d",(*iter) );
        r += temp;    //??????????
    }
    r += " } ";//????????????
    r += PlayerAction2String(playerAct) + ")";
    return "";
}




int WECoach::SentDefineMsg()
{
    double  left,top,right,bottom;
    char    temp[1024],str[1024];
    string  token;
    int     count=0;
    FILE    *f;

    if( (f = fopen(DEFINE_PARA,"r") ) != NULL)
    {
        token = "(define";
        while(fgets(temp,1024,f) != NULL)
        {
            if(temp[0]!='#')
            {
                switch(temp[0])
                {
                    case 'r':
                        sscanf(temp,"%s%s%d",str,str,&rowNum);
                        break;
                    case 'v':
                        sscanf(temp,"%s%s%d",str,str,&volNum);
                        break;
                    case '(':
                        token += " ";
                        token += temp;
                        count++;
                        break;
                    default:
                        break;
                }
                if(count%40||count==59)
                {
                    token += ")";
                    Say(token);
                    //cout<<token<<endl;
                    token = "(define";
                }
            }
        }
        fclose(f);
    }
    else return -1;
    
    rowWidth = FIELD_WIDTH / rowNum;
    volWidth = FIELD_LENGTH / volNum;
    //cout<<rowWidth<<" "<<volWidth<<endl;


    token = "(define";
    for(int x=0;x<volNum;x++)
        for(int y=0;y<rowNum;y++)
        {
            //(definer <NAME> <REGION>)
            left = x * volWidth - FIELD_LENGTH/2;
            right = (x+1) * volWidth - FIELD_LENGTH/2;
            top = y * rowWidth - FIELD_WIDTH/2;
            bottom = (y+1) * rowWidth - FIELD_WIDTH/2;
            sprintf(temp," (definer \"%c%c\" (rec (pt %0.1f %0.1f) (pt %0.1f %0.1f)))",
                            (x>9?(x+87):(x+48)), (y>9?(y+87):(y+48)), left,top, right,bottom);
            token += temp;
            if( (x*rowNum+y)%40 == 0 && (x+y) != 0 || x == (volNum-1) && y == (rowNum-1) )
            {
                token += ")";
                Say(token);
                token = "(define";
            }
        }
        
    if( (f = fopen(DEFINE_PARA,"r") ) != NULL)
    {
        while(fgets(temp,1024,f) != NULL)
        {
            if(temp[0]!='#')
            {
                switch(temp[0])
                {
                    case 'r':
                        sscanf(temp,"%s%s%d",str,str,&rowNum);
                        break;
                    case 'v':
                        sscanf(temp,"%s%s%d",str,str,&volNum);
                        break;
                    default:
                        break;
                }
            }
        }
        fclose(f);
    }
    else return -1;
    
    
    return 0;
}




//*********************************************************************************
//          Update Team Model
//*********************************************************************************

// Update formation stat
int WECoach::UpdateFormationStat()
{
    //l->Log(6,"update the opponent position model");
    om.UpdatePlayerPositionModel(wm.GetOppSide(), wm.GetCurrentTime(), UPDATE_FORMATION_PERIOD);

    // set the goalie number
    if( !bGoalieSet && wm.IsOppGoalieNumSet() && wm.IsOurGoalieNumSet() )
    {
        mm.f.goalie = wm.GetOurGoalieNum();
        om.f.goalie = wm.GetOppGoalieNum();
        bGoalieSet = true;
    }

    // Update the expected formation
    if(wm.GetCurrentTime()>=UPDATE_FORMATION_INTERVAL
        && (wm.GetCurrentTime()%UPDATE_FORMATION_INTERVAL)==0 )
    {
        l->Log(6,"at time %d update the formation stat",wm.GetCurrentTime());
        mm.UpdatePlayerExpectedPosition(wm.GetOurSide(), wm.GetCurrentTime(), UPDATE_FORMATION_PERIOD);
        om.UpdatePlayerExpectedPosition(wm.GetOppSide(), wm.GetCurrentTime(), UPDATE_FORMATION_PERIOD);
    }

    return 0;
}


// Update the ball possessing situation
int WECoach::UpdateBallPossessionState()
{
    lastBallPossessionState = currentBallPossessionState;
    currentBallPossessionState = GetBallPossessionState(wm.GetCurrentTime());
    return 0;
}


// record a player's Max speed for future decision
int WECoach::UpdateMaxSpeed()
{
    for(int i=0;i<MAX_PLAYER_NUMBER;i++)
    {
        int ctm = wm.GetCurrentTime();
        if(wm.GetTeammateSpeed(i+1,ctm)>mm.p[i].GetMaxSpeed())
            mm.p[i].SetMaxSpeed(wm.GetTeammateSpeed(i+1,ctm));
        if(wm.GetOpponentSpeed(i+1,ctm)>om.p[i].GetMaxSpeed())
            om.p[i].SetMaxSpeed(wm.GetOpponentSpeed(i+1,ctm));
    }
    return 0;
}

int WECoach::UpdateHoldStat()
{
    return 0;
}

int WECoach::UpdatePassStat()
{
    return 0;
}

int WECoach::UpdateDribbleStat()
{
    return 0;
}

int WECoach::UpdateShootStat()
{
    return 0;
}



// This function scane the field records ,
// recognize individual behavior and teamwork , 
// then Update our & opponent team models 
int WECoach::UpdateTeamModel()
{
    UpdateMaxSpeed();
    UpdateBallPossessionState();

    //UpdateHoldStat();
    //UpdatePassStat();
    //UpdateDribbleStat();
    //UpdateShootStat();
    
    UpdateFormationStat();
    
    return 0;
}





//*********************************************************************************
//          Analyze 
//*********************************************************************************


// analyze the player role
int WECoach::AnalyzePlayerRole(TeamT t, int uniformNumber)
{
    Team *tm;
    if( t == TT_OUR )
        tm = &mm;
    else
        tm = &om;
    int num = uniformNumber - 1;

    if( uniformNumber == tm->f.goalie )
    {
        tm->p[num].SetRole( PT_GOALIE );
        return 0;
    }

    VecPos playerExpPos = tm->f.expectedFormation[num];
    VecPos frontestPlayerPos;
    VecPos tailestPlayerPos;
    VecPos leftestPlayerPos;
    VecPos rightestPlayerPos;
    double centerYThreshold, midY, maxY=-100, minY=100;
    double forwardXThreshold,backXThreshold,minX=100, maxX=-100;
    
    double signY;
    
    for(int i=0;i<MAX_PLAYER_NUMBER;i++)
    {
        if( (i+1) != tm->f.goalie )
        {
            midY += tm->f.expectedFormation[i].GetY();
            if(tm->f.expectedFormation[i].GetY()>maxY) maxY = tm->f.expectedFormation[i].GetY();
            if(tm->f.expectedFormation[i].GetY()<minY) minY = tm->f.expectedFormation[i].GetY();
            if(tm->f.expectedFormation[i].GetX()>maxX) maxX = tm->f.expectedFormation[i].GetX();
            if(tm->f.expectedFormation[i].GetX()<minX) minX = tm->f.expectedFormation[i].GetX();
        }
    }
    midY /= ( MAX_PLAYER_NUMBER - 1 );
    centerYThreshold = (maxY-minY) * CENTER_Y_THRESHOLD;
    forwardXThreshold = (maxX-minX) * FORWARD_X_DIFF_THRESHOLD;
    backXThreshold = (maxX-minX) * BACK_X_DIFF_THRESHOLD;
    
    if( tm->GetSide(t) == FIELD_SIDE_L )
    {
        frontestPlayerPos = tm->f.rightestPlayerPos;
        tailestPlayerPos = tm->f.leftestPlayerPos;
        leftestPlayerPos = tm->f.topestPlayerPos;
        rightestPlayerPos = tm->f.bottomestPlayerPos;
        signY = 1;
    }
    else if( tm->GetSide(t) == FIELD_SIDE_R )
    {
        frontestPlayerPos = tm->f.leftestPlayerPos;
        tailestPlayerPos = tm->f.rightestPlayerPos;
        leftestPlayerPos = tm->f.bottomestPlayerPos;
        rightestPlayerPos = tm->f.topestPlayerPos;
        signY = -1;
    }
    else return -1;
    
    if(fabs(playerExpPos.GetX() - frontestPlayerPos.GetX())<=forwardXThreshold)
    {//forward
        if(fabs(playerExpPos.GetY() - midY)<=centerYThreshold)
        {
            tm->p[num].SetRole( PT_FORWARD_CENTER );
            tm->f.forwardCenter.insert(uniformNumber);
            tm->f.forwardNum++;
        }
        else if( (playerExpPos.GetY() - midY) * signY > 0 )
        {
            tm->p[num].SetRole( PT_FORWARD_RIGHT );
            tm->f.forwardRight.insert(uniformNumber);
            tm->f.forwardNum++;
        }
        else if( (playerExpPos.GetY() - midY) *signY < 0 )
        {
            tm->p[num].SetRole( PT_FORWARD_LEFT );
            tm->f.forwardLeft.insert(uniformNumber);
            tm->f.forwardNum++;
        }
        else tm->p[num].SetRole( PT_NONE );
    }
    else if(fabs(playerExpPos.GetX() - tailestPlayerPos.GetX())<=backXThreshold)
    {//backward
        if(fabs(playerExpPos.GetY() - midY)<=centerYThreshold)
        {
            tm->p[num].SetRole( PT_BACK_CENTER );
            tm->f.backCenter.insert(uniformNumber);
            tm->f.backNum++;
        }
        else if( (playerExpPos.GetY() - midY) * signY > 0 )
        {
            tm->p[num].SetRole( PT_BACK_RIGHT );
            tm->f.backRight.insert(uniformNumber);
            tm->f.backNum++;
        }
        else if( (playerExpPos.GetY() - midY) * signY < 0 )
        {
            tm->p[num].SetRole( PT_BACK_LEFT );
            tm->f.backLeft.insert(uniformNumber);
            tm->f.backNum++;
        }
        else tm->p[num].SetRole( PT_NONE );
    }
    else
    {//middlefield
        if(fabs(playerExpPos.GetY() - midY)<=centerYThreshold)
        {
            tm->p[num].SetRole( PT_MIDFIELD_CENTER );
            tm->f.midfieldCenter.insert(uniformNumber);
            tm->f.middleNum++;
        }
        else if( (playerExpPos.GetY() - midY) * signY > 0 )
        {
            tm->p[num].SetRole( PT_MIDFIELD_RIGHT );
            tm->f.midfieldRight.insert(uniformNumber);
            tm->f.middleNum++;
        }
        else if( (playerExpPos.GetY() - midY) * signY < 0 )
        {
            tm->p[num].SetRole( PT_MIDFIELD_LEFT );
            tm->f.midfieldLeft.insert(uniformNumber);
            tm->f.middleNum++;
        }
        else tm->p[num].SetRole( PT_NONE );
    }
    
    return 0;
}


// analyze the player roles
int WECoach::AnalyzePlayerRoles()
{
    if(wm.GetCurrentTime()>UPDATE_FORMATION_INTERVAL)// && 
        //!wm.GetCurrentTime()%MIN_UPDATE_FORMATION_INTERVAL)
    {
        mm.f.forwardLeft.clear();
        mm.f.forwardCenter.clear();
        mm.f.forwardRight.clear();
        mm.f.midfieldLeft.clear();
        mm.f.midfieldCenter.clear();
        mm.f.midfieldRight.clear();
        mm.f.backLeft.clear();
        mm.f.backCenter.clear();
        mm.f.backRight.clear();
        mm.f.forwardNumLast = mm.f.forwardNum;
        mm.f.middleNumLast = mm.f.middleNum;
        mm.f.backNumLast = mm.f.backNum;
        mm.f.forwardNum = 0;
        mm.f.middleNum = 0;
        mm.f.backNum = 0;
    
        om.f.forwardLeft.clear();
        om.f.forwardCenter.clear();
        om.f.forwardRight.clear();
        om.f.midfieldLeft.clear();
        om.f.midfieldCenter.clear();
        om.f.midfieldRight.clear();
        om.f.backLeft.clear();
        om.f.backCenter.clear();
        om.f.backRight.clear();
        om.f.forwardNumLast = om.f.forwardNum;
        om.f.middleNumLast = om.f.middleNum;
        om.f.backNumLast = om.f.backNum;
        om.f.forwardNum = 0;
        om.f.middleNum = 0;
        om.f.backNum = 0;

        for(int i=0;i<MAX_PLAYER_NUMBER;i++)
        {
            AnalyzePlayerRole(TT_OUR, i+1);
            AnalyzePlayerRole(TT_OPP, i+1);
        };
        bPlayerRolesSet = true;
    }
    
    return 0;
}


BallPossessionT WECoach::GetBallPossessionState(int tm)
{
    // check players with ball
    vector<int> mateWithBall = wm.GetOurBallOwner(tm);
    vector<int> oppWithBall = wm.GetOppBallOwner(tm);
    
    // update the players' ball control record
    for(int i=0;i<MAX_PLAYER_NUMBER;i++)
    {
        bool in = false;
        int size = mateWithBall.size();
        for(int j=0;j<size;j++)
        {
            if( (i+1) == mateWithBall[j] )
            {
                if(  mm.p[i].possessionRecord.size() 
                        && !mm.p[i].possessionRecord[mm.p[i].possessionRecord.size()-1].set )
                {//do nothing to continue count the control time
                }
                else
                {
                    BallPossessionStruct tempBPS;
                    tempBPS.startTime = wm.GetCurrentTime();
                    tempBPS.set = false;
                    mm.p[i].possessionRecord.push_back(tempBPS);
                }
                in = true;
                break;
            }
        }
        if( !in 
            && mm.p[i].possessionRecord.size() 
            && !mm.p[i].possessionRecord[mm.p[i].possessionRecord.size()-1].set )
        {
            mm.p[i].possessionRecord[mm.p[i].possessionRecord.size()-1].set = true;
            mm.p[i].possessionRecord[mm.p[i].possessionRecord.size()-1].endTime = wm.GetCurrentTime();
            mm.p[i].possessionRecord[mm.p[i].possessionRecord.size()-1].possessionTime =
                mm.p[i].possessionRecord[mm.p[i].possessionRecord.size()-1].endTime - 
                mm.p[i].possessionRecord[mm.p[i].possessionRecord.size()-1].startTime;
        }
    }

    for(int i=0;i<MAX_PLAYER_NUMBER;i++)
    {
        bool in = false;
        int size = mateWithBall.size();
        for(int j=0;j<size;j++)
        {
            if( (i+1) == mateWithBall[j] )
            {
                if( om.p[i].possessionRecord.size()
                    && !om.p[i].possessionRecord[om.p[i].possessionRecord.size()-1].set )
                {//do nothing to continue count the control time
                }
                else
                {
                    BallPossessionStruct tempBPS;
                    tempBPS.startTime = wm.GetCurrentTime();
                    tempBPS.set = false;
                    om.p[i].possessionRecord.push_back(tempBPS);
                }
                in = true;
                break;
            }
        }
        if( !in 
            && om.p[i].possessionRecord.size()
            && !om.p[i].possessionRecord[om.p[i].possessionRecord.size()-1].set )
        {
            om.p[i].possessionRecord[om.p[i].possessionRecord.size()-1].set = true;
            om.p[i].possessionRecord[om.p[i].possessionRecord.size()-1].endTime = wm.GetCurrentTime();
            om.p[i].possessionRecord[om.p[i].possessionRecord.size()-1].possessionTime =
                om.p[i].possessionRecord[om.p[i].possessionRecord.size()-1].endTime - 
                om.p[i].possessionRecord[om.p[i].possessionRecord.size()-1].startTime;
        }
    }
    
    // update ball possession state
    if( mateWithBall.size() > oppWithBall.size() )
        return BP_OUR;
    else if( mateWithBall.size() < oppWithBall.size() )
        return BP_OPP;
    else if( mateWithBall.size() )
    {
        switch( lastBallPossessionState )
        {
            case BP_OUR:
                return BP_OUR;
                break;
            case BP_OPP:
                return BP_OPP;
                break;
            default:
                return lastBallPossessionState;
        }
    }
    else switch( lastBallPossessionState )
        {
            case BP_OUR:
                return BP_OUR_TO_NONE;
                break;
            case BP_OPP:
                return BP_OPP_TO_NONE;
                break;
            default:
                return lastBallPossessionState;
        }
}


int WECoach::AnalyzeBallPossession(BallPossessionT bps)
{
    if(bps == BP_NONE) return -1;
    
    // record the possession information
    int ctm = wm.GetCurrentTime();
    if(bps == BP_OUR_TO_OPP)
    {
        if( mm.possessionRecord.size() )
        {
            if( !mm.possessionRecord[mm.possessionRecord.size()-1].set )
            {
                mm.possessionRecord[mm.possessionRecord.size()-1].endTime = ctm;
                mm.possessionRecord[mm.possessionRecord.size()-1].possessionTime 
                    = ctm - mm.possessionRecord[mm.possessionRecord.size()-1].startTime + 1;
                mm.possessionRecord[mm.possessionRecord.size()-1].set = true;
            }
        }
        BallPossessionStruct tempBPS;
        tempBPS.startTime = ctm;
        tempBPS.set = false;
        om.possessionRecord.push_back(tempBPS);
    }
    else
    {
        if( om.possessionRecord.size() )
        {
            if( !om.possessionRecord[om.possessionRecord.size()-1].set )
            {
                om.possessionRecord[om.possessionRecord.size()-1].endTime = ctm;
                om.possessionRecord[om.possessionRecord.size()-1].possessionTime 
                   = ctm - om.possessionRecord[om.possessionRecord.size()-1].startTime;
                om.possessionRecord[om.possessionRecord.size()-1].set = true;
            }
        }
        BallPossessionStruct tempBPS;
        tempBPS.startTime = ctm;
        tempBPS.set = false;
        mm.possessionRecord.push_back(tempBPS);
    }


    return 0;
}


int WECoach::AnalyzeTeammateAction()
{
    l->Log(6,"enter the AnalyzeTeammateAction()");

    // analyze our players' action during the 
    // lastest control ball period
    int startTime, endTime, time, size = mm.possessionRecord.size();
    vector<int> ballOwner;
    int ballControler,ballControlerLast;
    int startControlTime, endControlTime;
    
    TeamActS    teamAct;
    //PlayerActS  playerAct;
    
    if( size )
    {
        startTime = mm.possessionRecord[size-1].startTime;
        endTime = mm.possessionRecord[size-1].endTime;
        time = startTime;
        
        ballOwner = wm.GetOurBallOwner(time);
        if(ballOwner.size())
        {
            ballControlerLast = 0;
            startControlTime = time;
            endControlTime = 0;
            teamAct.organizer = ballOwner[0];
            teamAct.set = true;
        }
        else
        {
            cout<<"error at time "<<time<<" : situation impossible!! -- no mate control ball"<<endl;
            return -1;//error,situation imposssible!!
        }
        
        while( time <= endTime )
        {
            l->Log(6,"enter the loop with time %d",time);

            if(ballOwner.size())
            {//some mate controle ball
                l->Log(6,"mate %d control ball",ballOwner[0]);
                //one of our player control the ball
                ballControler = ballOwner[0];
                if(ballControler != ballControlerLast)
                {//mate controling ball change
                    //if last teamAct is not set then backfill the teamAct
                    if(!teamAct.set)
                    {
                        if(ballControler == teamAct.organizer)
                        {
                            if(teamAct.mateAct[teamAct.organizer].actType == PAT_DRIBBLE)
                            {
                                //dribble success
                                for(int i=0;i<MAX_PLAYER_NUMBER;i++)
                                {
                                    //mate i's action
                                    teamAct.mateAct[i].actRegion.left = wm.GetTeammatePositionX(i+1,time) + REGION_LEFT_BUF;
                                    teamAct.mateAct[i].actRegion.right = wm.GetTeammatePositionX(i+1,time) + REGION_RIGHT_BUF;
                                    teamAct.mateAct[i].actRegion.top = wm.GetTeammatePositionY(i+1,time) + REGION_TOP_BUF;
                                    teamAct.mateAct[i].actRegion.bottom = wm.GetTeammatePositionY(i+1,time) + REGION_BOTTOM_BUF;
                                }
                                teamAct.utility = wm.GetTeamActUtility(teamAct);
                            }
                            else
                            {
                                //action maybe fail,amend the prejudgement or deny the action
                                for(int i=0;i<MAX_PLAYER_NUMBER;i++)
                                {
                                    //mate i's action
                                    teamAct.mateAct[i].actRegion.left = wm.GetTeammatePositionX(i+1,time) + REGION_LEFT_BUF;
                                    teamAct.mateAct[i].actRegion.right = wm.GetTeammatePositionX(i+1,time) + REGION_RIGHT_BUF;
                                    teamAct.mateAct[i].actRegion.top = wm.GetTeammatePositionY(i+1,time) + REGION_TOP_BUF;
                                    teamAct.mateAct[i].actRegion.bottom = wm.GetTeammatePositionY(i+1,time) + REGION_BOTTOM_BUF;
                                }
                                teamAct.utility = wm.GetTeamActUtility(teamAct);
                            }
                        }
                        else 
                        {//pass
                            if(ballControler == teamAct.mateAct[teamAct.organizer].actMate)
                            {
                                //action success
                                for(int i=0;i<MAX_PLAYER_NUMBER;i++)
                                {
                                    //mate i's action
                                    teamAct.mateAct[i].actRegion.left = wm.GetTeammatePositionX(i+1,time) + REGION_LEFT_BUF;
                                    teamAct.mateAct[i].actRegion.right = wm.GetTeammatePositionX(i+1,time) + REGION_RIGHT_BUF;
                                    teamAct.mateAct[i].actRegion.top = wm.GetTeammatePositionY(i+1,time) + REGION_TOP_BUF;
                                    teamAct.mateAct[i].actRegion.bottom = wm.GetTeammatePositionY(i+1,time) + REGION_BOTTOM_BUF;
                                }
                                teamAct.utility = wm.GetTeamActUtility(teamAct);
                            }
                            else
                            {
                                //action maybe fail , amend the prejudgement or deny the action
                                for(int i=0;i<MAX_PLAYER_NUMBER;i++)
                                {
                                    //mate i's action
                                    teamAct.mateAct[i].actRegion.left = wm.GetTeammatePositionX(i+1,time) + REGION_LEFT_BUF;
                                    teamAct.mateAct[i].actRegion.right = wm.GetTeammatePositionX(i+1,time) + REGION_RIGHT_BUF;
                                    teamAct.mateAct[i].actRegion.top = wm.GetTeammatePositionY(i+1,time) + REGION_TOP_BUF;
                                    teamAct.mateAct[i].actRegion.bottom = wm.GetTeammatePositionY(i+1,time) + REGION_BOTTOM_BUF;
                                }
                                teamAct.utility = wm.GetTeamActUtility(teamAct);
                            }
                        }
                        
                        //check the record uniqueness, remove the old judgement
                        if(mm.possessionRecord[size-1].actionQueue.size()>0
                            && mm.possessionRecord[size-1].actionQueue[mm.possessionRecord[size-1].actionQueue.size()-1].time>=teamAct.time)
                            mm.possessionRecord[size-1].actionQueue.pop_back();
                        //record the judgement
                        l->Log(6,"set a team action :");
                        l->Log(6,"time == %d",teamAct.time);
                        l->Log(6,"organizer == %d",teamAct.organizer);
                        l->Log(6,"organizer's action == %s",teamAct.mateAct[teamAct.organizer-1].actType==PAT_DRIBBLE?"dribble":(teamAct.mateAct[teamAct.organizer-1].actType==PAT_PASS?"pass":(teamAct.mateAct[teamAct.organizer-1].actType==PAT_SHOOT?"shoot":"none")) );
                        l->Log(6,"the action utility == %f",teamAct.utility);
                        mm.possessionRecord[size-1].actionQueue.push_back(teamAct);
                        teamAct.set = true;
                    }
                    
                    l->Log(6,"set the teamAct atributes");
                    //this is the first cycle this mate control ball
                    //or there is a very short distant pass
                    //last team action end
                    ballControlerLast = ballControler;
                    startControlTime = time;
                    endControlTime = time;
                    
                    //record the situation and the team action at this moment
                    teamAct.time = startControlTime;
                    teamAct.playMode = wm.GetPlayMode(teamAct.time);
                    //record ball condition
                    teamAct.ballPos.left = wm.GetBallPositionX(teamAct.time) + REGION_LEFT_BUF;
                    teamAct.ballPos.right = wm.GetBallPositionX(teamAct.time) + REGION_RIGHT_BUF;
                    teamAct.ballPos.top = wm.GetBallPositionY(teamAct.time) + REGION_TOP_BUF;
                    teamAct.ballPos.bottom = wm.GetBallPositionY(teamAct.time) + REGION_BOTTOM_BUF;
                    //record the organizer
                    teamAct.organizer = ballControler;
                    //record player position condition
                    for(int i=0;i<MAX_PLAYER_NUMBER;i++)
                    {
                        //mate i's condition
                        teamAct.matePos[i].left = wm.GetTeammatePositionX(i+1,teamAct.time) + REGION_LEFT_BUF;
                        teamAct.matePos[i].right = wm.GetTeammatePositionX(i+1,teamAct.time) + REGION_RIGHT_BUF;
                        teamAct.matePos[i].top = wm.GetTeammatePositionY(i+1,teamAct.time) + REGION_TOP_BUF;
                        teamAct.matePos[i].bottom = wm.GetTeammatePositionY(i+1,teamAct.time) + REGION_BOTTOM_BUF;
                        //opp i's condition
                        teamAct.matePos[i].left = wm.GetOpponentPositionX(i+1,teamAct.time) + REGION_LEFT_BUF;
                        teamAct.matePos[i].right = wm.GetOpponentPositionX(i+1,teamAct.time) + REGION_RIGHT_BUF;
                        teamAct.matePos[i].top = wm.GetOpponentPositionY(i+1,teamAct.time) + REGION_TOP_BUF;
                        teamAct.matePos[i].bottom = wm.GetOpponentPositionY(i+1,teamAct.time) + REGION_BOTTOM_BUF;
                        //action
                        if( (i+1)==ballControler)
                        {
                            teamAct.mateAct[i].actType = PAT_DRIBBLE;
                            teamAct.mateAct[i].actMate = 0;
                        }
                        else
                        {
                            teamAct.mateAct[i].actType = PAT_POS;
                            teamAct.mateAct[i].actMate = 0;
                        }
                    }
                }
                else ;//controler continue control the ball
            }
            else
            {//no mate control ball
                l->Log(6,"no mate control ball");
                ballOwner = wm.GetOppBallOwner(time);
                if(ballOwner.size())
                {
                    //our control end , the ball has been reach by opp ,
                    //so the lastest action failed.
                    if(!teamAct.set)
                    {
                        teamAct.utility = -1;//wm.GetTeamActUtility(teamAct);
                        //check the record uniqueness, remove the old judgement
                        if(mm.possessionRecord[size-1].actionQueue.size()>0
                            && mm.possessionRecord[size-1].actionQueue[mm.possessionRecord[size-1].actionQueue.size()-1].time>=teamAct.time)
                            mm.possessionRecord[size-1].actionQueue.pop_back();
                        //record the judgement
                        mm.possessionRecord[size-1].actionQueue.push_back(teamAct);
                        teamAct.set = true;
                    }
                    else
                    {
                        teamAct.utility = -1;//wm.GetTeamActUtility(teamAct);
                        //check the record uniqueness, remove the old judgement
                        if(mm.possessionRecord[size-1].actionQueue.size()>0
                            && mm.possessionRecord[size-1].actionQueue[mm.possessionRecord[size-1].actionQueue.size()-1].time>=teamAct.time)
                            mm.possessionRecord[size-1].actionQueue.pop_back();
                        //record the judgement
                        mm.possessionRecord[size-1].actionQueue.push_back(teamAct);
                    }
                    l->Log(6,"opp %d control ball, our this control end at time %d",ballOwner[0],time);
                }
                else
                {
                    if(endControlTime <= startControlTime)
                    {
                        //this team action ends and another team action excluding close dribble is executing.
                        //record mates' action
                        for(int i=0;i<MAX_PLAYER_NUMBER;i++)
                        {
                            //mate i's action
                            teamAct.mateAct[i].actRegion.left = wm.GetTeammatePositionX(i+1,time) + REGION_LEFT_BUF;
                            teamAct.mateAct[i].actRegion.right = wm.GetTeammatePositionX(i+1,time) + REGION_RIGHT_BUF;
                            teamAct.mateAct[i].actRegion.top = wm.GetTeammatePositionY(i+1,time) + REGION_TOP_BUF;
                            teamAct.mateAct[i].actRegion.bottom = wm.GetTeammatePositionY(i+1,time) + REGION_BOTTOM_BUF;
                        }
                        teamAct.utility = wm.GetTeamActUtility(teamAct);
                        //check the record uniqueness, remove the old judgement
                        if(mm.possessionRecord[size-1].actionQueue.size()>0
                            && mm.possessionRecord[size-1].actionQueue[mm.possessionRecord[size-1].actionQueue.size()-1].time>=teamAct.time)
                            mm.possessionRecord[size-1].actionQueue.pop_back();
                        //record the judgement
                        mm.possessionRecord[size-1].actionQueue.push_back(teamAct);
                        endControlTime = time;
                        ballControlerLast = 0;
                        
                        //analyze the controler's intention that is one part of the new team action
                        teamAct.time = time-1;
                        teamAct.playMode = wm.GetPlayMode(teamAct.time);
                        //record ball condition
                        teamAct.ballPos.left = wm.GetBallPositionX(teamAct.time) + REGION_LEFT_BUF;
                        teamAct.ballPos.right = wm.GetBallPositionX(teamAct.time) + REGION_RIGHT_BUF;
                        teamAct.ballPos.top = wm.GetBallPositionY(teamAct.time) + REGION_TOP_BUF;
                        teamAct.ballPos.bottom = wm.GetBallPositionY(teamAct.time) + REGION_BOTTOM_BUF;
                        //record player position condition
                        for(int i=0;i<MAX_PLAYER_NUMBER;i++)
                        {
                            //mate i's condition
                            teamAct.matePos[i].left = wm.GetTeammatePositionX(i+1,teamAct.time) + REGION_LEFT_BUF;
                            teamAct.matePos[i].right = wm.GetTeammatePositionX(i+1,teamAct.time) + REGION_RIGHT_BUF;
                            teamAct.matePos[i].top = wm.GetTeammatePositionY(i+1,teamAct.time) + REGION_TOP_BUF;
                            teamAct.matePos[i].bottom = wm.GetTeammatePositionY(i+1,teamAct.time) + REGION_BOTTOM_BUF;
                            //opp i's condition
                            teamAct.matePos[i].left = wm.GetOpponentPositionX(i+1,teamAct.time) + REGION_LEFT_BUF;
                            teamAct.matePos[i].right = wm.GetOpponentPositionX(i+1,teamAct.time) + REGION_RIGHT_BUF;
                            teamAct.matePos[i].top = wm.GetOpponentPositionY(i+1,teamAct.time) + REGION_TOP_BUF;
                            teamAct.matePos[i].bottom = wm.GetOpponentPositionY(i+1,teamAct.time) + REGION_BOTTOM_BUF;
                            //action
                            if( (i+1)==ballControler)
                            {
                                //identify the controler is shooting , passing or dribbling.
                                teamAct.mateAct[i] = wm.GetTeammateIntention(ballControler,time);
                            }
                            else
                            {
                                teamAct.mateAct[i].actType = PAT_POS;
                                teamAct.mateAct[i].actMate = 0;
                            }
                            //record the new action's organizer
                            teamAct.organizer = ballControler;
                            teamAct.set = false;
                        }
                        l->Log(6,"mate %d release ball at %d GetTeammateIntention(%d , %d)",ballControler,time,ballControler,time);
                    }
                    else
                    {
                        //need to do nothing
                    }
                }
            }
            
            time++;
            if(time<=wm.GetCurrentTime())
                ballOwner = wm.GetOurBallOwner(time);
        }
    }
    return 0;
}

int WECoach::AnalyzeOpponentAction()
{
    // analyze opponent players' action during the 
    // lastest control ball period
    int startTime, endTime, time, size = om.possessionRecord.size();
    
    if( size )
    {
        startTime = om.possessionRecord[size-1].startTime;
        endTime = om.possessionRecord[size-1].endTime;
        time = startTime;
        while( time < endTime )
        {
            time++;
        }
    }
    return 0;
}

int WECoach::AnalyzePlayerAction(BallPossessionT bps)
{
    if(bps==BP_NONE)
        return 0;
    else if(bps==BP_OUR_TO_OPP)
        return AnalyzeTeammateAction();
    else if(bps==BP_OPP_TO_OUR)
        return AnalyzeOpponentAction();
    else
        return 0;
}


int WECoach::Analyze()
{
    AnalyzePlayerRoles();

    AnalyzeBallPossession( IsBallPossessionStateChange() );
    
    AnalyzePlayerAction( IsBallPossessionStateChange() );

    return 0;
}




//*********************************************************************************
//          Supervise 
//*********************************************************************************

// supervise the current plan
// and calculate the utility and cost
int WECoach::Supervise()
{
    string sRuleOn = "";
    string sRuleOff = "";
    //string sRuleDelete = "";
    
    for(int i=superviseRuleMark;i<proccessRuleMark;i++)
    {
        if(ruleRecordQ[i].sent)
        {
            if(ruleRecordQ[i].on)
            {
                if(ruleRecordQ[i].onTime < (wm.GetCurrentTime()-ADVICE_LIVE_TIME_NORMAL) )
                {
                    //turn rule off
                    sRuleOff += " " + ruleRecordQ[i].ruleName;
                    //sRuleDelete += " " + ruleRecordQ[i].ruleName;
                    ruleRecordQ[i].on = false;
                    superviseRuleMark = i+1;
                }
                else ;//do nothing
            }
            else
            {
                //turn rule on
                sRuleOn += " " + ruleRecordQ[i].ruleName;
                ruleRecordQ[i].on = true;
            }
        }
    }
    
    Command cmd;
    if(sRuleOn!="")
    {
        cmd.type = CT_SAY;
        cmd.c = "(rule (on (" + sRuleOn + ")))";
        cmd.createTime = wm.GetCurrentTime();
        sendMsgQ.push_back(cmd);
    }
    if(sRuleOff!="")
    {
        cmd.type = CT_SAY;
        cmd.c = "(rule (off (" + sRuleOff + ")))";
        cmd.createTime = wm.GetCurrentTime();
        sendMsgQ.push_back(cmd);
    }
    return 0;
}





//*********************************************************************************
//          Decision 
//*********************************************************************************

// Decide our teammates' type
int WECoach::DecidePlayerType()
{
    // if the role of teammates have been observed
    // then our coach will change the player type for our team
    //PlayModeT pm = wm.GetPlayMode(wm.GetCurrentTime());

    if(bPlayerRolesSet 
        //&& (pm != PM_PLAY_ON)
        && !bDecidePlayerType
      )
    {
        Command cmd;
        cmd.type = CT_COMMAND;
        set<int>::iterator  iter;
        int unum;
        
        for( iter=mm.f.forwardCenter.begin(); iter!=mm.f.forwardCenter.end(); iter++)
        {
            unum = *iter;
            cmd.c = ChangePlayerTypeCommand(unum,pts.SelectForwardCenter());
            cmd.createTime = wm.GetCurrentTime();
            sendMsgQ.push_back(cmd);
l->Log(6,"change center %d to type",unum);
        }
        for( iter=mm.f.forwardLeft.begin(); iter!=mm.f.forwardLeft.end(); iter++)
        {
            unum = *iter;
            cmd.c = ChangePlayerTypeCommand(unum,pts.SelectForwardSide());
            cmd.createTime = wm.GetCurrentTime();
            sendMsgQ.push_back(cmd);
l->Log(6,"change left %d to type",unum);
        }
        for( iter=mm.f.forwardRight.begin(); iter!=mm.f.forwardRight.end(); iter++)
        {
            unum = *iter;
            cmd.c = ChangePlayerTypeCommand(unum,pts.SelectForwardSide());
            cmd.createTime = wm.GetCurrentTime();
            sendMsgQ.push_back(cmd);
l->Log(6,"change right %d to type",unum);
        }
        bDecidePlayerType = true;
    }
    
    return 0;
}


bool WECoach::NeedToDecideNewPosition()
{
    if(wm.GetCurrentTime()<=MIN_INTERVAL) return false;
    
    if( (wm.GetCurrentTime()-1)%MIN_INTERVAL!=0) return false;
    
    int controlTime = mm.GetControlBallTime();
    l->Log(8,"the our control time is %d",controlTime);
    float controlDiff = mm.GetControlBallRatio();
    l->Log(8,"the our control ratio is %f",controlDiff);

    return true;
}



FormationT WECoach::GetFormation(TeamT tm)
{
    Team *t;
    if(tm==TT_OUR)
        t = &mm;
    else if(tm==TT_OPP)
        t = &om;
    else
        return FT_NONE;

    FormationT formationType;// = FT_433;
    int forwardNum = t->f.forwardLeft.size() + t->f.forwardCenter.size() + t->f.forwardRight.size();
    int midfieldNum = t->f.midfieldLeft.size() + t->f.midfieldCenter.size() + t->f.midfieldRight.size();
    int backNum = t->f.backLeft.size() + t->f.backCenter.size() + t->f.backRight.size();
    
    if(backNum==4 && midfieldNum==3 && forwardNum==3)
        formationType = FT_433;
    else if(backNum==4 && midfieldNum==2 && forwardNum==4)
        formationType = FT_424;
    else if(backNum==4 && midfieldNum==4 && forwardNum==2)
        formationType = FT_442;
    else if(backNum==4 && midfieldNum==5 && forwardNum==1)
        formationType = FT_451;
    else if(backNum==3 && midfieldNum==5 && forwardNum==2)
        formationType = FT_352;
    else if(backNum==3 && midfieldNum==4 && forwardNum==3)
        formationType = FT_343;
    else if(backNum==3 && midfieldNum==3 && forwardNum==4)
        formationType = FT_334;
    else if(backNum==5 && midfieldNum==3 && forwardNum==2)
        formationType = FT_532;
    else
        formationType = FT_0;
        
    l->Log(10,"forward line player number == %d",forwardNum);
    l->Log(10,"midfield line player number == %d",midfieldNum);
    l->Log(10,"back line player number == %d",backNum);
    l->Log(10,"the formation is %d",formationType);
        
    //fill in the player role map table
    int i=0;
    set<int>::iterator iter;

    t->role2PlayerMap[i] = t->f.goalie;
    i++;
    //if(t->GetSide(tm)==FIELD_SIDE_L)
    //{
        //back
        for( iter=t->f.backRight.begin(); iter!=t->f.backRight.end(); iter++)
        {
            t->role2PlayerMap[i] = *iter;
            //l->Log(10,"the %d player is %d",i,*iter);
            i++;
        }
        for( iter=t->f.backCenter.begin(); iter!=t->f.backCenter.end(); iter++)
        {
            t->role2PlayerMap[i] = *iter;
            //l->Log(10,"the %d player is %d",i,*iter);
            i++;
        }
        for( iter=t->f.backLeft.begin(); iter!=t->f.backLeft.end(); iter++)
        {
            t->role2PlayerMap[i] = *iter;
            //l->Log(10,"the %d player is %d",i,*iter);
            i++;
        }
        //midfield
        for( iter=t->f.midfieldRight.begin(); iter!=t->f.midfieldRight.end(); iter++)
        {
            t->role2PlayerMap[i] = *iter;
            //l->Log(10,"the %d player is %d",i,*iter);
            i++;
        }
        for( iter=t->f.midfieldCenter.begin(); iter!=t->f.midfieldCenter.end(); iter++)
        {
            t->role2PlayerMap[i] = *iter;
            //l->Log(10,"the %d player is %d",i,*iter);
            i++;
        }
        for( iter=t->f.midfieldLeft.begin(); iter!=t->f.midfieldLeft.end(); iter++)
        {
            t->role2PlayerMap[i] = *iter;
            //l->Log(10,"the %d player is %d",i,*iter);
            i++;
        }
        //forward
        for( iter=t->f.forwardRight.begin(); iter!=t->f.forwardRight.end(); iter++)
        {
            t->role2PlayerMap[i] = *iter;
            //l->Log(10,"the %d player is %d",i,*iter);
            i++;
        }
        for( iter=t->f.forwardCenter.begin(); iter!=t->f.forwardCenter.end(); iter++)
        {
            t->role2PlayerMap[i] = *iter;
            //l->Log(10,"the %d player is %d",i,*iter);
            i++;
        }
        for( iter=t->f.forwardLeft.begin(); iter!=t->f.forwardLeft.end(); iter++)
        {
            t->role2PlayerMap[i] = *iter;
            //l->Log(10,"the %d player is %d",i,*iter);
            i++;
        }

		return formationType;
}



// Decide our teammates' position
int WECoach::DecidePositioning()
{
	//add your positioning model here

    return 0;
}


void WECoach::DecideTeamplay()
{
    int size = mm.possessionRecord.size();
    //l->Log(6,"checkPossessionRecordIndex = %d ; mm.possessionRecord.size() = %d",checkPossessionRecordIndex,size);
    if( (checkPossessionRecordIndex+1) >= size) return;

    //check our possession record and action record
    //and decide to continue the teamplay or stop the teamplay
    vector<BallPossessionStruct>::iterator iterPossession;
    vector<TeamActS>::iterator iterAction;
    Command cmd;
    string condition, direction, liveTime;
    char tempC[2048];
    string tempS;
    vector<int> unumSet;
    VecPos ballPos;
    VecPos playerPos;

    int index = -1;
    for(iterPossession = mm.possessionRecord.begin() ; iterPossession!=mm.possessionRecord.end(); iterPossession++ )
    {
        index++;
        if(index<=checkPossessionRecordIndex)
        {
        }
        else if( (*iterPossession).actionQueue.size()>0)
        {
            l->Log(6,"sending the new advice....%d...",(*iterPossession).actionQueue.size());
            for(iterAction=(*iterPossession).actionQueue.begin(); iterAction!=(*iterPossession).actionQueue.end(); iterAction++)
            {
                //cout<<(*iterAction).utility<<endl;
                //l->Log(6,"utility == %f",(*iterAction).utility);
                if( (*iterAction).utility > NICE_TEAMPLAY_THRESHOLD)
                {
                    cmd.type = CT_DEFINE;
                
                    //create live time
                    //sprintf(tempC,"definerule %s direc (",GetRuleName);
                    //liveTime = tempC;
                    ballPos.SetVect( ((*iterAction).ballPos.left+(*iterAction).ballPos.right)/2, ((*iterAction).ballPos.top+(*iterAction).ballPos.bottom)/2);
                    
                    //create condition , that are the position of ball and players
                    condition = "(and";
                    tempS = PlayMode2String( (*iterAction).playMode );  //(playm <PLAY_MODE>)
                    condition += " " + tempS;                           //(and (playm <PLAY_MODE>).
                    tempS = BallPos2String( (*iterAction).ballPos );    //(bpos <REGION>)
                    condition += " " + tempS; //(and (playm <PLAY_MODE>) (bpos <REGION>).
                    for(int i=0;i<MAX_PLAYER_NUMBER;i++)
                    {
                        //calculate if the mate position is relevant
                        //to be added
                        playerPos.SetVect( ((*iterAction).matePos[i].left+(*iterAction).matePos[i].right)/2, ((*iterAction).matePos[i].top+(*iterAction).matePos[i].bottom)/2);
                        if(playerPos.GetDistanceTo(ballPos)<NEAR_THRESHOLD )
                        {//mate's position is relevant
                            unumSet.clear();
                            unumSet.push_back(i+1);
                            tempS = PlayerPos2String(TT_OUR, unumSet, 1, 1, (*iterAction).matePos[i] );//(ppos <TEAM> <UNUM_SET> <INT> <INT> <REGION>)
                            condition += " " + tempS;
                        }
                        //calculate if the mate position is relevant
                        //to be added
                        playerPos.SetVect( ((*iterAction).oppPos[i].left+(*iterAction).oppPos[i].right)/2, ((*iterAction).oppPos[i].top+(*iterAction).oppPos[i].bottom)/2);
                        if(playerPos.GetDistanceTo(ballPos)<NEAR_THRESHOLD)
                        {//opponent's position is relevant
                            unumSet.clear();
                            unumSet.push_back(i+1);
                            tempS = PlayerPos2String(TT_OPP, unumSet, 1, 1, (*iterAction).oppPos[i] );//(ppos <TEAM> <UNUM_SET> <INT> <INT> <REGION>)
                            condition += " " + tempS;
                        }
                    }
                    condition += ")";
                    
                    //create direction , that are the action of players
                    direction = "";
                    for(int i=0;i<MAX_PLAYER_NUMBER;i++)
                    {
                        //calculate if the mate's action is relevant
                        playerPos.SetVect( ((*iterAction).matePos[i].left+(*iterAction).matePos[i].right)/2, ((*iterAction).matePos[i].top+(*iterAction).matePos[i].bottom)/2);
                        if(playerPos.GetDistanceTo(ballPos)<ACT_NEAR_THRESHOLD )
                        {//mate's action is relevant
                            direction += " (do our ";          //(<MODE> <TEAM> <UNUM_SET> <ACTION_LIST>)
                            sprintf(tempC,"{%d} ",i+1);
                            direction += tempC;
                            tempS = PlayerAction2String( (*iterAction).mateAct[i]);//(pass <REGION>)|(pos <REGION>)|(dribble <REGION>)|(shoot)
                            direction += tempS + ")";
                        }
                    }   
    
                    //create command
                    cmd.c = "(definerule " + GetRuleName() + " direc (" + condition + direction +"))";
                    cmd.createTime = wm.GetCurrentTime();
                    sendMsgQ.push_back(cmd);
                }   
                else
                    if( (*iterAction).utility < ACCEPTABLE_TEAMPLAY_THRESHOLD)
                    {
                        //cmd.type = CT_DEFINE;
                        //create condition
                        //create direction
                        //cmd.c = "(" + liveTime + " " + condition + " " + direction +")";
                        //sendMsgQ.push_back(cmd);
                    }
                    else
                    {
                        //cout<<"%%%"<<endl;
                        //l->Log(6,"This teamplay is neither nice nor bad need to be added.");
                    }
            }   
            checkPossessionRecordIndex++;
        }
    }
    
    //create new teamplay
    //to be added
}


int WECoach::DecideTeachTeamplay()
{
    int tm = wm.GetCurrentTime();
    string token;
    Command cmd;
    cmd.type = CT_DEFINE;
    
    vector<string> tempSV;
    
    if(tm>MIN_INTERVAL)
    {
		//add your countermeasure here
    }
    return 0;
}

// Do attack decision for our team
int WECoach::DoAttackDecision()
{
    DecidePlayerType();
    DecidePositioning();
    DecideTeamplay();
    DecideTeachTeamplay();
    return 0;
}

// Decide our mark distribution
int WECoach::DecideMark()
{
	//add your mark distribution here
	
    return 0;
}

// Do defend decision for our team
int WECoach::DoDefendDecision()
{
    DecideMark();
    return 0;
}


// sent opponent information to our players
int WECoach::DoInfoDecision()
{
    if(wm.GetCurrentTime()> (2*UPDATE_FORMATION_INTERVAL)
        && wm.GetPlayMode(wm.GetCurrentTime())!=PM_PLAY_ON )
    {
        //to be added
        //send opponent position model
    }
    return 0;
}


//*****************************************
// This Is the paramount decision process
//*****************************************
int WECoach::Decision(bool online)
{
    if( lastDecisionTime >= wm.GetCurrentTime() ) return -1;

l->Log(2,"Decision at time %d :",wm.GetCurrentTime());

    UpdateTeamModel();
    Analyze();

    if(online){
        //onlinecoach
        Supervise();
        DoAttackDecision();
        DoDefendDecision();
        DoInfoDecision();
        SendCommand();
    }else{
	//offlinecoach
	//add your online coach (trainer)'s 
	//scene testing, setting functions here.
	//Sample commands:
	//Send("(change_mode free_kick_l");
	//Send("(move (ball) 0 0)");
	//Send("(move (player teamname unum) 0 0 0 0 0 0)");
    }

l->Log(2,"End time %d decision\n\n",wm.GetCurrentTime());

    lastDecisionTime = wm.GetCurrentTime();
    return 0;
}






//*********************************************************************************
//          Memory and judge functions 
//*********************************************************************************

vector<string> WECoach::GenOurTactic(FormationT fm,Tactic tac,int LifeSpan)
{
	vector<string> rv;

	int LLB,LMB,MB,RMB,RRB; // 

	int LLM,LMM,MM,RMM,RRM; // г

	int LLF,LMF,MF,RMF,RRF; //ǰ

	LLB = LMB = MB = RMB = RRB = -1;
	LLM = LMM = MM = RMM = RRM = -1;
	LLF = LMF = MF = RMF = RRF = -1;
	
        switch(fm){
		case FT_433:
			break;
		case FT_442:
			break;
		case FT_352:
			break;
		case FT_532:
			break;
	    default:
	        return rv;
	        break;
		}


	string r;

	switch(tac){
	case TA_Left:
		break;
	case TA_Right:
		break;
	case TA_Middle:
		break;
	case TA_Corner:
		break;
	case TA_CTS: // ·ֱ
		break;
	case TA_STC: // б·
		break;
	case TA_COF: // Խλ...
		break;
	case TA_BCLOSS: // бת
		break;
	default:
		break;
	}

	return rv;
}










//*********************************************************************************
//          Memory and judge functions 
//*********************************************************************************

BallPossessionT WECoach::IsBallPossessionStateChange()
{
    if( lastBallPossessionState == BP_OUR && currentBallPossessionState == BP_OPP  || 
        lastBallPossessionState == BP_OUR_TO_NONE && currentBallPossessionState == BP_OPP ||
        lastBallPossessionState == BP_NONE && currentBallPossessionState == BP_OPP)
        return BP_OUR_TO_OPP;
    else if ( lastBallPossessionState == BP_OPP && currentBallPossessionState == BP_OUR ||
                lastBallPossessionState == BP_OPP_TO_NONE && currentBallPossessionState == BP_OUR ||
                lastBallPossessionState == BP_NONE && currentBallPossessionState == BP_OUR)
        return BP_OPP_TO_OUR;
    else 
        return BP_NONE;
}


string WECoach::GetRuleName()
{
    string res;
    char name[1024];
    ruleName++;
    sprintf(name,"R%d",ruleName);
    res = name;
    RuleRecord rr;
    rr.ruleName = res;
    //rr.onTime = wm.GetCurrentTime();
    ruleRecordQ.push_back(rr);
    return res;
}




//*********************************************************************************
//          For WE only 
//*********************************************************************************

int WECoach::ChangeWEPlayerType()
{
	return 0;
}


